@@ -110,19 +110,24 @@ class ParentPart(object):
         # self.name is only set if a part is also a widget
         self.name=None
 
+        # dictionary of labels to values to be inserted into javascript as fieldvalues attribute
+        # again, only used if a part is a widget
+        self.jlabels = {}
+
         self.tag_name = tag_name
-        self._attribs = {}
+
         self.hide_if_empty = False
 
         # set when set_idents is called on the containing page
         self.ident_string = ''
 
-
         # self.brief is a string describing what this part does
         self.brief = brief
 
         if attribs:
-            self.set_attribs(attribs)
+            self.attribs = attribs
+        else:
+            self.attribs = {}
 
         # set show to False if this part is not to be shown
         self.show = show
@@ -196,47 +201,50 @@ class ParentPart(object):
         if not self.ident_string:
             return ''
         # ident_string normally starts project_pageident_head-0-... etc
-        return self.ident_string.split('_')[0] ##############################
+        return self.ident_string.split('_')[0]
 
     def insert_id(self, id_string=''):
         """Adds the id_string to the part attributes. If no id_string given
            but the part has a name, sets the id to the part name, otherwise
-           sets the id to self.ident_string"""
+           sets the id to self.ident_string, returns the id"""
         if id_string:
-            self.update_attribs({"id": id_string})
-            return
+            self.attribs["id"] = id_string
+            return id_string
         if self.name:
             # This only applies to named widgets
             if self.placename:
                 full_id = self.placename + '-' + self.name
-                self.update_attribs({"id": full_id})
+                self.attribs["id"] = full_id
+                return full_id
             else:
-                self.update_attribs({"id": self.name})
-            return
+                self.attribs["id"] = self.name
+                return self.name
         if self.ident_string:
-            self.update_attribs({"id": self.ident_string})
+            self.attribs["id"] = self.ident_string
+            return self.ident_string
 
     def get_id(self):
-        return self.get_attrib_value('id')
+        "Return id, or None if not set"
+        return self.attribs.get('id')
+
 
     def get_class(self):
-        return self.get_attrib_value('class')
+        "Return class, or None if not set"
+        return self.attribs.get('class')
 
     def set_class(self, value):
-        if (not value) and ('class' in self._attribs):
-            del self._attribs['class']
+        if (not value) and ('class' in self.attribs):
+            del self.attribs['class']
         else:
-            self._attribs['class'] = value
+            self.attribs['class'] = value
 
     def set_hide(self, value=True):
         "If True, Sets display:none !important into style, if False removes it"
         if value:
-            attribs = self._attribs
-            if 'style' not in attribs:
-                self._attribs['style'] = "display:none !important;"
+            style = self.attribs.get('style')
+            if not style:
+                self.attribs['style'] = "display:none !important;"
                 return
-            # An existing style is in place
-            style = self._attribs['style']
             style_list = style.split(';')
             # ensure no empty items
             style_list = [item for item in style_list if item]
@@ -254,17 +262,15 @@ class ParentPart(object):
                 # "display:block !important" already set, remove it so it can be replaced
                 style_list.remove("display:block !important")
             # append display:none !important;
-            if not style_list:
-                self._attribs['style'] = "display:none !important;"
-                return
-            style_list.append("display:none !important;")
-            self._attribs['style'] = ";".join(style_list)
+            if style_list:
+                style_list.append("display:none !important;")
+                self.attribs['style'] = ";".join(style_list)
+            else:
+                self.attribs['style'] = "display:none !important;"
         else:
-            attribs = self._attribs
-            if 'style' not in attribs:
+            style = self.attribs.get('style')
+            if not style:
                 return
-            # An existing style is in place
-            style = self._attribs['style']
             style_list = style.split(';')
             style_list = [item for item in style_list if item]
             if "display:none !important" in style_list:
@@ -272,18 +278,16 @@ class ParentPart(object):
             if "display:none" in style_list:
                 style_list.remove("display:none")
             if style_list:
-                self._attribs['style'] = ";".join(style_list)
+                self.attribs['style'] = ";".join(style_list)
             else:
-                del self._attribs['style']
+                del self.attribs['style']
 
     def set_block(self):
         "Sets display:block !important into style"
-        attribs = self._attribs
-        if 'style' not in attribs:
-            self._attribs['style'] = "display:block !important;"
+        style = self.attribs.get('style')
+        if not style:
+            self.attribs['style'] = "display:block !important;"
             return
-        # An existing style is in place
-        style = self._attribs['style']
         style_list = style.split(';')
         # ensure no empty items
         style_list = [item for item in style_list if item]
@@ -301,34 +305,18 @@ class ParentPart(object):
             # "display:none !important" already set, remove it so it can be replaced
             style_list.remove("display:none !important")
         # append display:block !important;
-        if not style_list:
-            self._attribs['style'] = "display:block !important;"
-            return
-        style_list.append("display:block !important;")
-        self._attribs['style'] = ";".join(style_list)
+        if style_list:
+            style_list.append("display:block !important;")
+            self.attribs['style'] = ";".join(style_list)
+        else:
+            self.attribs['style'] = "display:block !important;"
 
-    def get_attribs(self):
-        return self._attribs.copy()
-
-    def set_attribs(self, attribs):
-        self._attribs = {}
-        if attribs:
-            for att,val in attribs.items():
-                if val:
-                    self._attribs[att]=val
-                else:
-                    self._attribs[att]=''
-
-    def del_attribs(self):
-        self._attribs = {}
-
-    attribs = property(get_attribs, set_attribs, del_attribs)
 
     @property
     def attributes_string(self):
         "Returns a string showing the attributes"
         str_attribs = ''
-        for att,val in self._attribs.items():
+        for att,val in self.attribs.items():
             if val:
                 if val[0] == '{':
                     value = self._expand_label(str(val))
@@ -356,26 +344,18 @@ class ParentPart(object):
                 return quote(url, safe='/:?&=@')+valstring[end+1:]
         return valstring
 
-    def has_attrib(self, name):
-        "If this attrib exists, return True otherwise False"
-        return name in self._attribs
 
-    def get_attrib_value(self, name):
-        """Get an attribute value, given its name, if it does not exist, return None"""
-        if name not in self._attribs:
-            return None
-        return self._attribs[name]
-
-    def del_one_attrib(self, attrib):
-        "deletes an attribute"
-        if attrib in self._attribs:
-            del self._attribs[attrib]
+    def set_class_style(self, class_attribute='', style_attribute=''):
+        "Sets the tag class and style attributes"
+        if class_attribute:
+            self.attribs["class"] = class_attribute
+        if style_attribute:
+            self.attribs["style"] = style_attribute
 
 
-    def update_attribs(self, attribs):
-        "Updates attributes with the dictionary given in the attribs argument"
-        for att,val in attribs.items():
-            self._attribs[att]=val
+    def get_url(self, label_url_ident):
+        "Returns a url given a page ident, url or label, if not found, return None"
+        return skiboot.get_url(label_url_ident, self.proj_ident)
 
 
     def make_get_url(self, page, url, get_fields={}, force_ident=False):
@@ -399,7 +379,7 @@ class ParentPart(object):
         """If a message is given, sets self._error
         """
         # update attribute with  data-status="error"
-        self.update_attribs({'data-status':'error'})
+        self.attribs['data-status'] = 'error'
         if message:
             self._error = message
         self.show = True
@@ -495,13 +475,13 @@ class Part(ParentPart):
     text = property(get_text, set_text, doc="Equivalent to self[0]; note may not be text despite this attributes name")
 
 
-    def set_contained_values(self, values):
+    def set_contained_values(self, *args, **kwargs):
         """This may be called during a _build by a widget wishing to set values into contained widgets
            It is generally overwitten by widgets which are intended to be contained, and need to do something
            with information from their parent container"""
         for index, part in enumerate(self.parts):
             if hasattr(part, "set_contained_values"):
-                part.set_contained_values(values)
+                part.set_contained_values(*args, **kwargs)
 
         
     def update(self, page, ident_list, environ, call_data, lang, ident_string, placename='', embedded=('','',None)):
@@ -939,7 +919,7 @@ class SectionPlaceHolder(object):
     def get_section(self):
         if not self.ident_string:
             return
-        proj_ident = self.ident_string.split('_')[0] ########################
+        proj_ident = self.ident_string.split('_')[0]
         proj = skiboot.getproject(proj_ident)
         if proj is None:
             return
