@@ -8,6 +8,7 @@ from tag.Part and tag.ClosedPart"""
 
 import html, copy, collections, re, json, datetime
 from string import Template
+from types import SimpleNamespace
 
 from .. import tag, skiboot
 from ..excepts import ServerError, ValidateError
@@ -522,51 +523,51 @@ class Widget(tag.Part):
     # if it is not None, then show_error will show the error at that location
     error_location = None
 
-    arg_descriptions = {'widget_class':FieldArg("cssclass", "", jsonset=True)}
+    # If True, this widget will only be shown if debug is True
+    only_show_on_debug = False
+
+    arg_descriptions = {}
+
+    common_args = {'widget_class':FieldArg("cssclass", "", jsonset=True),
+                   'widget_style':FieldArg("cssstyle", "", jsonset=True),
+                   'show':FieldArg("boolean", True)}
+
+    error_args = {'show_error':FieldArg("text", '', jsonset=True),
+                  'clear_error':FieldArg("boolean", False, jsonset=True )}
+
+
 
     # All widgets automatically have a show, show_error, widget_class and widget_style field arguments
 
-    def __init__(self, name=None, tag_name="div", brief='', **field_args):
+    def __init__(self, name=None, brief='', **field_args):
 
         if not brief:
             brief = self.__class__.__name__
 
-        tag.Part.__init__(self, tag_name=tag_name, text='', show=True, brief=brief, hide_if_empty=False)
+        tag.Part.__init__(self, tag_name="div", text='', show=True, brief=brief, hide_if_empty=False)
         self.name = name
-        # create self.fields which is a dictionary of {field_arg:FieldArg,...}
-        # for example - in this case field_arg is the string 'widget_class', and value is
-        # a deep copy of the FieldArg object created by FieldArg("cssclass", "", jsonset=True)
-        self.fields = copy.deepcopy(self.arg_descriptions)
-        # ALL Widgets have a show and widget_class arguments
-        if 'show' not in self.fields:
-            self.fields['show'] = FieldArg("boolean", True)
-        if 'widget_class' not in self.fields:
-            self.fields['widget_class'] = FieldArg("cssclass", '', jsonset=True)
-        if 'widget_style' not in self.fields:
-            self.fields['widget_style'] = FieldArg("cssstyle", '', jsonset=True)
-        # if widget has display_errors set True, then they also have a show_error and clear_error argument
-        if self.display_errors:
-            if 'show_error' not in self.fields:
-                self.fields['show_error'] = FieldArg("text", '', jsonset=True)
-            if 'clear_error' not in self.fields:
-                self.fields['clear_error'] = FieldArg("boolean", False, jsonset=True )
+        # create self.fields which is a dictionary of {fname:FieldArg,...}, by calling the class method
+        # field_name_dict() which gets copies of arg_descriptions, common_args and error_args and returns them
+        # as the single dictionary fields
+        self.fields = copy.deepcopy(self.field_name_dict())
         # Each FieldArg object has a name and value.
-        # **field_args is a dictionary of {field_arg:value, ...}  - in this example it could be
-        # {"widget_class": "css_class_name',...}
-        # For each FieldArg object in self.fields, if its field_arg does not appear in the **field_args, then its value will be that
+ 
+        # **field_args is a dictionary of {fname:value, ...} 
+
+        # For each FieldArg object in self.fields, if its fname does not appear in the **field_args, then its value will be that
         # given in the class attribute 'default'. If it does appear, then its
         # value is set to that given in **field_args
-        for field_arg, field in self.fields.items():
-            # initially set field names to be the same as field argument
+        for fname, field in self.fields.items():
+            # initially set field names to be the same as given in arg_descriptions
             if self.name:
-                field.name = field_arg
+                field.name = fname
             else:
                 # If this widget has no name - it is set inside another. To avoid name clashes when inserting one widget in another,
-                # all names in this widget are initially set to start with an underscore"
-                field.name = '_' + field_arg
-            # set field values to be the values given in **field_args (note: field_args is plural to differentiate)
-            if field_arg in field_args:
-                field.value = field_args[field_arg]
+                # all names in this widget are initially set to start with an underscore
+                field.name = '_' + fname
+            # set field values to be the values given in **field_args
+            if fname in field_args:
+                field.value = field_args[fname]
         # set initial self.error_message to the value given in show_error argument
         if self.display_errors and self.fields['show_error'].value:
             self.error_message = self.fields['show_error'].value
@@ -576,19 +577,20 @@ class Widget(tag.Part):
         # the widget show is set by the show argument
         self.show = self.fields["show"].value
 
-        # Creates a dictionary of names against field arguments in self._names
+        # Creates a dictionary of names against field arguments fname in self._names
         self._create_name_dict()
 
         # If the widget is in error status, this becomes True
         self.error_status = False
 
+
     @classmethod
     def module_name(cls):
         return cls.__module__.split('.')[-1]
 
     def _create_name_dict(self):
-        "Creates a dictionary of names against field arguments"
-        self._names = { field.name:field_arg for field_arg, field in self.fields.items() }
+        "Creates a dictionary of names against field argument fnames"
+        self._names = { field.name:fname for fname, field in self.fields.items() }
 
 
     def get_container_parts(self, index):
@@ -740,25 +742,29 @@ class Widget(tag.Part):
             return None, None
 
 
+    @classmethod
+    def field_name_dict(cls):
+        "Returns a dictionary of {fname: fieldarg} as given in the class field description attributes"
+        # ALL Widgets have a widget_class, widget_style, show arguments
+        fields = cls.common_args.copy()
+        # if widget has display_errors set True, then fields also contains show_error and clear_error arguments
+        if cls.display_errors:
+            fields.update(cls.error_args)
+        # update with fields described in arg_descriptions
+        fields.update(cls.arg_descriptions)
+        return fields
+
+
     @classmethod
     def field_arguments_single(cls):
         """Returns a list of lists.
         The inner list consists of: [ field arg, field type, valdt, jsonset, cssclass, cssstyle]
         sorted by field argument"""
         args = []
-        for arg, item in cls.arg_descriptions.items():
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArg):
-                args.append( [arg, item.field_type, item.valdt, item.jsonset, item.cssclass, item.cssstyle] )
-        if 'show' not in cls.arg_descriptions:
-            args.append( ['show', 'boolean', False, False, False, False] )
-        if 'widget_class' not in cls.arg_descriptions:
-            args.append( ['widget_class', 'cssclass', False, True, True, False] )
-        if 'widget_style' not in cls.arg_descriptions:
-            args.append( ['widget_style', 'cssstyle', False, True, False, True] )
-        if cls.display_errors and ('show_error' not in cls.arg_descriptions):
-            args.append( ['show_error', 'text', False, True, False, False] )
-        if cls.display_errors and ('clear_error' not in cls.arg_descriptions):
-            args.append( ['clear_error', 'boolean', False, True, False, False] )
+                args.append( [fname, item.field_type, item.valdt, item.jsonset, item.cssclass, item.cssstyle] )
         args.sort(key=lambda row: row[0])
         return args
 
@@ -768,13 +774,13 @@ class Widget(tag.Part):
         """Returns a list of lists.
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument"""
-        arg_list = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgList):
-                arg_list.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_list:
-            arg_list.sort(key=lambda row: row[0])
-        return arg_list
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
@@ -783,13 +789,13 @@ class Widget(tag.Part):
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument.
         field type is a list of column types"""
-        arg_table = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgTable):
-                arg_table.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_table:
-            arg_table.sort(key=lambda row: row[0])
-        return arg_table
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
@@ -797,31 +803,19 @@ class Widget(tag.Part):
         """Returns a list of lists.
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument"""
-        arg_dict = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgDict):
-                arg_dict.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_dict:
-            arg_dict.sort(key=lambda row: row[0])
-        return arg_dict
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
     def field_arguments(cls):
         """Returns a list of field args"""
-        args = []
-        for arg in cls.arg_descriptions:
-            args.append(arg)
-        if 'show' not in cls.arg_descriptions:
-            args.append('show')
-        if 'widget_class' not in cls.arg_descriptions:
-            args.append('widget_class')
-        if 'widget_style' not in cls.arg_descriptions:
-            args.append('widget_style')
-        if cls.display_errors and ('show_error' not in cls.arg_descriptions):
-            args.append('show_error')
-        if cls.display_errors and ('clear_error' not in cls.arg_descriptions):
-            args.append('clear_error')
+        args = list(cls.field_name_dict().keys())
         args.sort()
         return args
 
@@ -865,6 +859,9 @@ class Widget(tag.Part):
 
     def update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded):
         """Runs self._build, then update all sub widgets"""
+        if self.only_show_on_debug and not skiboot.get_debug():
+            self.show = False
+            return
         if not self.show:
             return
         if placename:
@@ -882,12 +879,10 @@ class Widget(tag.Part):
             if isinstance(self._error, TextBlock):
                 self._error.update(page, ident_list, environ, call_data, lang, self.ident_string, self.placename, embedded_parts)
             return
-        # build the widget
-        # the class attribute is set by 'widget_class'
-        if self.fields['widget_class'].value:
-            self.update_attribs({'class':self.fields['widget_class'].value})
-        if self.fields['widget_style'].value:
-            self.update_attribs({'style':self.fields['widget_style'].value})
+        # build the widget, set self.wf with field values, this namespace is used in the widget build to access widget field values easily
+        self.wf = SimpleNamespace(**{fname:item.value for fname,item in self.fields.items()})
+        # set the widget class and style attributes
+        self.set_class_style(self.wf.widget_class, self.wf.widget_style)
         # Insert this widgets id
         self.insert_id()
         # insert further parts according to each widget build
@@ -903,18 +898,6 @@ class Widget(tag.Part):
                 e.ident_list = ident_list
             raise
 
-
-    def _make_fieldvalues(self, *fieldargs, **otherparams):
-        "Creates a javascript string of fieldvalues, which can be used by _build_js"
-        fieldvalues = {}
-        for farg in fieldargs:
-            fieldvalues[farg] = self.get_field_value(farg)
-        fieldvalues.update(otherparams)
-        if not fieldvalues:
-            return ''
-        return """  SKIPOLE.widgets["{ident}"].fieldvalues={fieldvalues};
-""".format(ident=self.get_id(), fieldvalues=json.dumps(fieldvalues))
-
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Called by make_js, and should be overwritten by widgets to set widget specific javascript
            into the page.  This javascript is added last, after all other widget updates are done, and
@@ -980,7 +963,11 @@ class Widget(tag.Part):
 
         # and add widget specific json
         contents = self._build_js(page, ident_list, environ, call_data, lang)
+        if self.jlabels:
+            page.add_javascript(f"""  SKIPOLE.widgets["{self.get_id()}"].fieldvalues={json.dumps(self.jlabels)};
+""")
         if contents:
+            # contents added after fieldvalues in case these contents use the fieldvalues
             page.add_javascript(contents)
 
 
@@ -1020,7 +1007,7 @@ class Widget(tag.Part):
         self.error_status = True
         # if this is a named widget update attribute with data-status="error"
         if self.name:
-            self.update_attribs({'data-status':'error'})
+            self.attribs['data-status'] = 'error'
         if not message:
             message = self.error_message
         if self.error_location is not None:
@@ -1127,18 +1114,7 @@ class Widget(tag.Part):
         field.value = value
         if name == self.fields["show"].name:
             self.show = bool(value)
-        if name == self.fields['widget_class'].name:
-            if value:
-                self.update_attribs({'class':value})
-            else:
-                self.del_one_attrib('class')
-        if name == self.fields['widget_style'].name:
-            if value:
-                self.update_attribs({'style':value})
-            else:
-                self.del_one_attrib('style')
-
-        
+       
 
     def set_single_multivalue(self, name, value):
         """If a field name is given with a -, then after the dash must be an index
@@ -1178,7 +1154,7 @@ class Widget(tag.Part):
             part = self.get_location_value(loc)
         else:
             part = self
-        part.del_attribs()
+        part.attribs = {}
         for idx in range(n):
             # field_arg is data1, data2 etc..
             field_arg = 'data' + str(idx+1)
@@ -1186,7 +1162,7 @@ class Widget(tag.Part):
                 raise ServerError("Field argument %s does not exist in this widget." % (field_arg,))
             if self.get_field_value(field_arg):
                 key = 'data-' + self.get_formname(field_arg)
-                part.update_attribs({key:str(self.get_field_value(field_arg))})
+                part.attribs[key] = str(self.get_field_value(field_arg))
 
     def set_field_value(self, field_arg, value):
         """If this widget contains fields, then this should set a field
@@ -1203,14 +1179,14 @@ class Widget(tag.Part):
             self.show = field.value
         if field_arg == 'widget_class':
             if value:
-                self.update_attribs({'class':value})
-            else:
-                self.del_one_attrib('class')
+                self.attribs['class'] = value
+            elif "class" in self.attribs:
+                del self.attribs["class"]
         if field_arg == 'widget_style':
             if value:
-                self.update_attribs({'style':value})
-            else:
-                self.del_one_attrib('style')
+                self.attribs['style'] = value
+            elif "style" in self.attribs:
+                del self.attribs["style"]
 
         if field_arg == 'show_error':
             self.error_message = field.value
@@ -1396,46 +1372,53 @@ class ClosedWidget(tag.ClosedPart):
     # display_errors is a class attribute, True if this widget accepts and displays error messages, False otherwise
     display_errors = True
 
+    # If True, this widget will only be shown if debug is True
+    only_show_on_debug = False
+
     # always empty list and unused
     _container=[]
 
     # always None and unused
     error_location = None
 
-    arg_descriptions = {'widget_class':FieldArg("cssclass", "", jsonset=True)}
+    arg_descriptions = {}
+
+    common_args = {'widget_class':FieldArg("cssclass", "", jsonset=True),
+                   'widget_style':FieldArg("cssstyle", "", jsonset=True),
+                   'show':FieldArg("boolean", True)}
+
+    error_args = {'show_error':FieldArg("text", '', jsonset=True),
+                  'clear_error':FieldArg("boolean", False, jsonset=True )}
 
 
-    def __init__(self, name=None, tag_name="link", brief='', **field_args):
+    def __init__(self, name=None, brief='', **field_args):
         if not brief:
             brief = self.__class__.__name__
-        tag.ClosedPart.__init__(self, tag_name=tag_name, show=True, brief=brief)
+        tag.ClosedPart.__init__(self, tag_name="link", show=True, brief=brief)
         self.name = name
-        # Get fields from arg_descriptions
-        self.fields = copy.deepcopy(self.arg_descriptions)
-        # ALL Widgets have a show, widget_class and widget_style arguments
-        if 'show' not in self.fields:
-            self.fields['show'] = FieldArg("boolean", True)
-        if 'widget_class' not in self.fields:
-            self.fields['widget_class'] = FieldArg("cssclass", "", jsonset=True)
-        if 'widget_style' not in self.fields:
-            self.fields['widget_style'] = FieldArg("cssstyle", '', jsonset=True)
-        # if widget has display_errors set True, then they also have a show_error and clear_error argument
-        if self.display_errors:
-            if 'show_error' not in self.fields:
-                self.fields['show_error'] = FieldArg("text", '', jsonset=True)
-            if 'clear_error' not in self.fields:
-                self.fields['clear_error'] = FieldArg("boolean", False, jsonset=True )
-        for field_arg, field in self.fields.items():
+        # create self.fields which is a dictionary of {fname:FieldArg,...}, by calling the class method
+        # field_name_dict() which gets copies of arg_descriptions, common_args and error_args and returns them
+        # as the single dictionary fields
+        self.fields = copy.deepcopy(self.field_name_dict())
+
+        # Each FieldArg object has a name and value.
+        # **field_args is a dictionary of {fname:value, ...} 
+
+        # For each FieldArg object in self.fields, if its fname does not appear in the **field_args, then its value will be that
+        # given in the class attribute 'default'. If it does appear, then its
+        # value is set to that given in **field_args
+
+        for fname, field in self.fields.items():
             # initially set field names to be the same as field argument
             if self.name:
-                field.name = field_arg
+                field.name = fname
             else:
                 # If this widget has no name - it is set inside another. To avoid name clashes when inserting one widget in another,
                 # all names in this widget are initially set to start with an underscore"
-                field.name = '_' + field_arg
+                field.name = '_' + fname
             # set field values to be the values given in **field_args
-            if field_arg in field_args:
-                field.value = field_args[field_arg]
+            if fname in field_args:
+                field.value = field_args[fname]
 
         # Creates a dictionary of names against field arguments in self._names
         self._create_name_dict()
@@ -1479,8 +1462,20 @@ class ClosedWidget(tag.ClosedPart):
             return None, None
 
     def _create_name_dict(self):
-        "Creates a dictionary of names against field arguments"
-        self._names = { field.name:field_arg for field_arg, field in self.fields.items() }
+        "Creates a dictionary of names against field arguments fnames"
+        self._names = { field.name:fname for fname, field in self.fields.items() }
+
+    @classmethod
+    def field_name_dict(cls):
+        "Returns a dictionary of {fname: fieldarg} as given in the class field description attributes"
+        # ALL Widgets have a widget_class, widget_style, show arguments
+        fields = cls.common_args.copy()
+        # if widget has display_errors set True, then fields also contains show_error and clear_error arguments
+        if cls.display_errors:
+            fields.update(cls.error_args)
+        # update with fields described in arg_descriptions
+        fields.update(cls.arg_descriptions)
+        return fields
 
 
     @classmethod
@@ -1489,19 +1484,10 @@ class ClosedWidget(tag.ClosedPart):
         The inner list consists of: [ field arg, field type, valdt, jsonset, cssclass, cssstyle]
         sorted by field argument"""
         args = []
-        for arg, item in cls.arg_descriptions.items():
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArg):
-                args.append( [arg, item.field_type, item.valdt, item.jsonset, item.cssclass, item.cssstyle] )
-        if 'show' not in cls.arg_descriptions:
-            args.append( ['show', 'boolean', False, False, False, False] )
-        if 'widget_class' not in cls.arg_descriptions:
-            args.append( ['widget_class', 'cssclass', False, True, True, False] )
-        if 'widget_style' not in cls.arg_descriptions:
-            args.append( ['widget_style', 'cssstyle', False, True, False, True] )
-        if cls.display_errors and ('show_error' not in cls.arg_descriptions):
-            args.append( ['show_error', 'text', False, True, False, False] )
-        if cls.display_errors and ('clear_error' not in cls.arg_descriptions):
-            args.append( ['clear_error', 'boolean', False, True, False, False] )
+                args.append( [fname, item.field_type, item.valdt, item.jsonset, item.cssclass, item.cssstyle] )
         args.sort(key=lambda row: row[0])
         return args
 
@@ -1511,13 +1497,13 @@ class ClosedWidget(tag.ClosedPart):
         """Returns a list of lists.
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument"""
-        arg_list = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgList):
-                arg_list.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_list:
-            arg_list.sort(key=lambda row: row[0])
-        return arg_list
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
@@ -1526,13 +1512,13 @@ class ClosedWidget(tag.ClosedPart):
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument.
         field type is a list of column types"""
-        arg_table = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgTable):
-                arg_table.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_table:
-            arg_table.sort(key=lambda row: row[0])
-        return arg_table
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
@@ -1540,31 +1526,19 @@ class ClosedWidget(tag.ClosedPart):
         """Returns a list of lists.
         The inner list consists of: [ field arg, field type, valdt, jsonset].
         sorted by field argument"""
-        arg_dict = []
-        for arg, item in cls.arg_descriptions.items():
+        args = []
+        fields = cls.field_name_dict()
+        for fname, item in fields.items():
             if isinstance(item, FieldArgDict):
-                arg_dict.append( [arg, item.field_type, item.valdt, item.jsonset] )
-        if arg_dict:
-            arg_dict.sort(key=lambda row: row[0])
-        return arg_dict
+                args.append( [fname, item.field_type, item.valdt, item.jsonset] )
+        args.sort(key=lambda row: row[0])
+        return args
 
 
     @classmethod
     def field_arguments(cls):
         """Returns a list of field args"""
-        args = []
-        for arg in cls.arg_descriptions:
-            args.append(arg)
-        if 'show' not in cls.arg_descriptions:
-            args.append('show')
-        if 'widget_class' not in cls.arg_descriptions:
-            args.append('widget_class')
-        if 'widget_style' not in cls.arg_descriptions:
-            args.append('widget_style')
-        if cls.display_errors and ('show_error' not in cls.arg_descriptions):
-            args.append('show_error')
-        if cls.display_errors and ('clear_error' not in cls.arg_descriptions):
-            args.append('clear_error')
+        args = list(cls.field_name_dict().keys())
         args.sort()
         return args
 
@@ -1605,6 +1579,9 @@ class ClosedWidget(tag.ClosedPart):
 
     def update(self, page, ident_list, environ, call_data, lang, ident_string, placename, embedded):
         """builds the widget"""
+        if self.only_show_on_debug and not skiboot.get_debug():
+            self.show = False
+            return
         if not self.show:
             return
         if placename:
@@ -1622,26 +1599,14 @@ class ClosedWidget(tag.ClosedPart):
             if isinstance(self._error, TextBlock):
                 self._error.update(page, ident_list, environ, call_data, lang, self.ident_string, self.placename, embedded_parts)
             return
-        # build the widget
-        # the class attribute is set by 'widget_class'
-        if self.fields['widget_class'].value:
-            self.update_attribs({'class':self.fields['widget_class'].value})
-        if self.fields['widget_style'].value:
-            self.update_attribs({'style':self.fields['widget_style'].value})
+        # build the widget, set self.wf with field values, this namespace is used in the widget build to access widget field values easily
+        self.wf = SimpleNamespace(**{fname:item.value for fname,item in self.fields.items()})
+        # set the widget class and style attributes
+        self.set_class_style(self.wf.widget_class, self.wf.widget_style)
         # Insert this widgets id
         self.insert_id()
         self._build(page, ident_list, environ, call_data, lang)
 
-    def _make_fieldvalues(self, *fieldargs, **otherparams):
-        "Creates a javascript string of fieldvalues, which can be used by _build_js"
-        fieldvalues = {}
-        for farg in fieldargs:
-            fieldvalues[farg] = self.get_field_value(farg)
-        fieldvalues.update(otherparams)
-        if not fieldvalues:
-            return ''
-        return """  SKIPOLE.widgets["{ident}"].fieldvalues={fieldvalues};
-""".format(ident=self.get_id(), fieldvalues=json.dumps(fieldvalues))
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         return ''
@@ -1704,10 +1669,14 @@ class ClosedWidget(tag.ClosedPart):
 
         # and add widget specific json
         contents = self._build_js(page, ident_list, environ, call_data, lang)
+        # add fieldvalues stored in self.jlabels
+        if self.jlabels:
+            page.add_javascript(f"""  SKIPOLE.widgets["{self.get_id()}"].fieldvalues={json.dumps(self.jlabels)};
+""")
         if contents:
+            # contents added after fieldvalues in case these contents use the fieldvalues
             page.add_javascript(contents)
 
-
     def set_placename(self, section_name, placename):
         "Widgets in sections with displayname validators need displaynames to change"
         self.placename = placename
@@ -1740,7 +1709,7 @@ class ClosedWidget(tag.ClosedPart):
         self.error_status = True
         # if this is a named widget update attribute with data-status="error"
         if self.name:
-            self.update_attribs({'data-status':'error'})
+            self.attribs['data-status'] = 'error'
         if not message:
             message = self.error_message
         self._error_build(message)
@@ -1835,17 +1804,6 @@ class ClosedWidget(tag.ClosedPart):
         field.value = value
         if name == self.fields["show"].name:
             self.show = bool(value)
-        if name == self.fields['widget_class'].name:
-            if value:
-                self.update_attribs({'class':value})
-            else:
-                self.del_one_attrib('class')
-        if name == self.fields['widget_style'].name:
-            if value:
-                self.update_attribs({'style':value})
-            else:
-                self.del_one_attrib('style')
-
 
     def set_single_multivalue(self, name, value):
         """If a field name is given with a -, then after the dash must be an index
@@ -1881,7 +1839,7 @@ class ClosedWidget(tag.ClosedPart):
             return
         if n != m:
             raise ServerError("field data values are mismatched")
-        self.del_attribs()
+        self.attribs = {}
         for idx in range(n):
             # field_arg is data1, data2 etc..
             field_arg = 'data' + str(idx+1)
@@ -1889,7 +1847,7 @@ class ClosedWidget(tag.ClosedPart):
                 raise ServerError("Field argument %s does not exist in this widget." % (field_arg,))
             if self.get_field_value(field_arg):
                 key = 'data-' + self.get_formname(field_arg)
-                self.update_attribs({key:str(self.get_field_value(field_arg))})
+                self.attribs[key] = str(self.get_field_value(field_arg))
 
     def set_field_value(self, field_arg, value):
         """If this widget contains fields, then this should set a field
@@ -1906,14 +1864,14 @@ class ClosedWidget(tag.ClosedPart):
             self.show = field.value
         if field_arg == 'widget_class':
             if value:
-                self.update_attribs({'class':value})
-            else:
-                self.del_one_attrib('class')
+                self.attribs['class'] = value
+            elif "class" in self.attribs:
+                del self.attribs["class"]
         if field_arg == 'widget_style':
             if value:
-                self.update_attribs({'style':value})
-            else:
-                self.del_one_attrib('style')
+                self.attribs['style'] = value
+            elif "style" in self.attribs:
+                del self.attribs["style"]
         if field_arg == 'show_error':
             self.error_message = field.value
 
@@ -2026,3 +1984,32 @@ class ClosedWidget(tag.ClosedPart):
                 part_dict["validators"] = collections.OrderedDict(sorted(val_dict.items(), key=lambda t: t[0]))
         return ['ClosedWidget', part_dict]
 
+
+class ClickEventMixin(object):
+
+    def _build_js(self, page, ident_list, environ, call_data, lang):
+        """Sets a click event handler on the widget"""
+        if 'id' not in self.attribs:
+            return ''
+        ident = self.attribs['id']
+        return f"""  $("#{ident}").click(function (e) {{
+    SKIPOLE.widgets['{ident}'].eventfunc(e);
+    }});
+"""
+
+
+class AnchorClickEventMixin(object):
+
+    def _build_js(self, page, ident_list, environ, call_data, lang):
+        """Sets a click event handler on the a tags in the widget"""
+        if 'id' not in self.attribs:
+            return ''
+        ident = self.attribs['id']
+        return f"""  $("#{ident} a").click(function (e) {{
+    SKIPOLE.widgets['{ident}'].eventfunc(e);
+    }});
+"""
+
+
+
+
