@@ -3,7 +3,7 @@
 """Contains widgets for dropdown forms"""
 
 
-from .. import skiboot, tag, excepts
+from .. import tag, excepts
 from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -39,7 +39,8 @@ class DropDown1(Widget):
         error_class: css class applied to the normally hidden error paragraph
         div_class: css class applied to the div after the error paragraph
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
         self[0][0][0] = ''
@@ -51,40 +52,31 @@ class DropDown1(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the checkbox"
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if self.get_field_value('div_class'):
-            self[1].update_attribs({"class":self.get_field_value('div_class')})
-        if self.get_field_value('left_label'):
-            self[1][0][0] = self.get_field_value('left_label')
-        if self.get_field_value('left_class'):
-            self[1][0].attribs = {"class": self.get_field_value('left_class')}
-        if self.get_field_value('left_style'):
-            self[1][0].attribs = {"style": self.get_field_value('left_style')}
-        if self.get_field_value('select_class'):
-            self[1][1].attribs = {"class": self.get_field_value('select_class')}
-        if self.get_field_value('select_style'):
-            self[1][1].attribs = {"style": self.get_field_value('select_style')}
-        self[1][1].update_attribs({"name":self.get_formname('selectvalue')})
-        selected_option = self.get_field_value('selectvalue')
-        for index, opt in enumerate(self.get_field_value('option_list')):
+            del self[0].attribs["style"]
+        self[1].set_class_style(self.wf.div_class)
+
+        if self.wf.left_label:
+            self[1][0][0] = self.wf.left_label
+        self[1][0].set_class_style(self.wf.left_class, self.wf.left_style)
+        self[1][1].set_class_style(self.wf.select_class, self.wf.select_style)
+        self[1][1].attribs["name"] = self.get_formname('selectvalue')
+
+        selected_option = self.wf.selectvalue
+        for index, opt in enumerate(self.wf.option_list):
             if selected_option == opt:
                 self[1][1][index] = tag.Part(tag_name="option", text=opt, attribs ={"selected":"selected"})
             else:
                 self[1][1][index] = tag.Part(tag_name="option", text=opt)
-        if self.get_field_value('right_label'):
-            self[1][2][0] = self.get_field_value('right_label')
-        if self.get_field_value('right_class'):
-            self[1][2].attribs = {"class": self.get_field_value('right_class')}
-        if self.get_field_value('right_style'):
-            self[1][2].attribs = {"style": self.get_field_value('right_style')}
+        if self.wf.right_label:
+            self[1][2][0] = self.wf.right_label
+        self[1][2].set_class_style(self.wf.right_class, self.wf.right_style)
         # set an id in the select for the 'label for' tag
-        self[1][1].insert_id()
+        for_id = self[1][1].insert_id()
         # set the label 'for' attribute
-        self[1][0].update_attribs({'for':self[1][1].get_id()})
-        self[1][2].update_attribs({'for':self[1][1].get_id()})
+        self[1][0].attribs['for'] = for_id
+        self[1][2].attribs['for'] = for_id
 
     @classmethod
     def description(cls):
@@ -153,7 +145,8 @@ class SubmitDropDown1(Widget):
         hide: If True, widget is hidden
         disabled: Set True if the select field and submit button are to be disabled
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # error para at 0
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -171,67 +164,64 @@ class SubmitDropDown1(Widget):
         # the submit button
         self[1][0][2] = tag.Part(tag_name="button", attribs ={"type":"submit"})
         self[1][0][2][0] = "Submit"
-        self._jsonurl = ''
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self.widget_hide(self.wf.hide)
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
-        # update the action of the form
-        self[1].update_attribs({"action": actionurl})
-        # the div holding label, dropdown and button
-        if self.get_field_value('inputdiv_class'):
-            self[1][0].attribs = {"class": self.get_field_value('inputdiv_class')}
-        if self.get_field_value('label'):
-            self[1][0][0][0] = self.get_field_value('label')
-        if self.get_field_value('label_class'):
-            self[1][0][0].attribs = {"class": self.get_field_value('label_class')}
-        if self.get_field_value('label_style'):
-            self[1][0][0].attribs = {"style": self.get_field_value('label_style')}
-        if self.get_field_value('select_class'):
-            self[1][0][1].attribs = {"class": self.get_field_value('select_class')}
-        if self.get_field_value('select_style'):
-            self[1][0][1].attribs = {"style": self.get_field_value('select_style')}
-        self[1][0][1].update_attribs({"name":self.get_formname('selectvalue')})
 
-        if self.get_field_value('disabled'):
-            self[1][0][1].update_attribs({"disabled":"disabled"})
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if self.wf.action_json:
+            self.jlabels['url'] = self.get_url(self.wf.action_json)
+
+        # update the action of the form
+        self[1].attribs["action"] = actionurl
+        # the div holding label, dropdown and button
+        self[1][0].set_class_style(self.wf.inputdiv_class)
+
+        if self.wf.label:
+            self[1][0][0][0] = self.wf.label
+        self[1][0][0].set_class_style(self.wf.label_class, self.wf.label_style)
+        self[1][0][1].set_class_style(self.wf.select_class, self.wf.select_style)
+
+        self[1][0][1].attribs["name"] = self.get_formname('selectvalue')
+
+        if self.wf.disabled:
+            self[1][0][1].attribs["disabled"] = "disabled"
 
         # set an id in the input field for the 'label for' tag
-        self[1][0][1].insert_id()
+        for_id = self[1][0][1].insert_id()
 
-        selected_option = self.get_field_value('selectvalue')
-        for index, opt in enumerate(self.get_field_value('option_list')):
+        selected_option = self.wf.selectvalue
+        for index, opt in enumerate(self.wf.option_list):
             if selected_option == opt:
                 self[1][0][1][index] = tag.Part(tag_name="option", text=opt, attribs ={"selected":"selected"})
             else:
                 self[1][0][1][index] = tag.Part(tag_name="option", text=opt)
 
         # set the label 'for' attribute
-        self[1][0][0].update_attribs({'for':self[1][0][1].get_id()})
+        self[1][0][0].attribs['for'] = for_id
 
         # submit button
-        if self.get_field_value('button_text'):
-            self[1][0][2][0] = self.get_field_value('button_text')
-        if self.get_field_value('button_class'):
-            self[1][0][2].update_attribs({"class": self.get_field_value('button_class')})
-        if self.get_field_value('disabled'):
-            self[1][0][2].update_attribs({"disabled":"disabled"})
+        if self.wf.button_text:
+            self[1][0][2][0] = self.wf.button_text
+
+        self[1][0][2].set_class_style(self.wf.button_class)
+        if self.wf.disabled:
+            self[1][0][2].attribs["disabled"] = "disabled"
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -239,14 +229,11 @@ class SubmitDropDown1(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
-        jscript = """  $('#{ident}').on("submit", function(e) {{
+        ident = self.get_id()
+        return f"""  $('#{ident}').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        if self._jsonurl:
-            return jscript + self._make_fieldvalues(url=self._jsonurl)
-        return jscript
-
+"""
 
     @classmethod
     def description(cls):
@@ -307,7 +294,8 @@ class HiddenContainer(Widget):
         inner_class: The CSS class of the div holding the paragraph
         buttondiv_class: The class of the div holding the button
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="div")
         # div holding X button
         self[0][0] = tag.Part(tag_name="div")
@@ -321,45 +309,38 @@ class HiddenContainer(Widget):
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the box"
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        if self.get_field_value("boxdiv_class"):
-            self[0].update_attribs({"class":self.get_field_value('boxdiv_class')})
-        if self.get_field_value('boxdiv_style'):
-            self[0].update_attribs({"style":self.get_field_value('boxdiv_style')})
+        self.widget_hide(self.wf.hide)
+ 
+        self[0].set_class_style(self.wf.boxdiv_class, self.wf.boxdiv_style)
         # buttondiv
-        if self.get_field_value("buttondiv_class"):
-            self[0][0].update_attribs({"class":self.get_field_value('buttondiv_class')})
-        if self.get_field_value("buttondiv_style"):
-            self[0][0].update_attribs({'style':self.get_field_value("buttondiv_style")})
+        self[0][0].set_class_style(self.wf.buttondiv_class, self.wf.buttondiv_style)
         # inner div
-        if self.get_field_value("inner_class"):
-            self[0][1].update_attribs({"class":self.get_field_value('inner_class')})
-        if self.get_field_value("inner_style"):
-            self[0][1].update_attribs({'style':self.get_field_value("inner_style")})
+        self[0][1].set_class_style(self.wf.inner_class, self.wf.inner_style)
         # button
-        if self.get_field_value('button_class'):
-            self[0][0][0].update_attribs({"class":self.get_field_value('button_class')})
-        if not self.get_field_value("link_ident"):
+        self[0][0][0].set_class_style(self.wf.button_class)
+
+        if not self.wf.link_ident:
             self[0][0][0][0] = "Warning: broken link"
         else:
-            url = skiboot.get_url(self.get_field_value("link_ident"), proj_ident=page.proj_ident)
+            url = self.get_url(self.wf.link_ident)
             if url:
                 # create a url for the href
-                get_fields = {self.get_formname("get_field1"):self.get_field_value("get_field1"),
-                              self.get_formname("get_field2"):self.get_field_value("get_field2"),
-                              self.get_formname("get_field3"):self.get_field_value("get_field3")}
+                get_fields = {self.get_formname("get_field1"):self.wf.get_field1,
+                              self.get_formname("get_field2"):self.wf.get_field2,
+                              self.get_formname("get_field3"):self.wf.get_field3}
                 url = self.make_get_url(page, url, get_fields, True)
-                self[0][0][0].update_attribs({"href": url})
+                self[0][0][0].attribs["href"] = url
             else:
                 self[0][0][0][0] = "Warning: broken link"
 
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a click event handler on the a button"""
-        return """  $("#{ident} a").first().click(function (e) {{
+        ident = self.get_id()
+        return f"""  $("#{ident} a").first().click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident = self.get_id())
+"""
 
 
     @classmethod
