@@ -2,7 +2,7 @@
 """Defines widgets of page headers,
 typically with titles, logo images and navigational buttons."""
 
-from .. import skiboot
+
 from .. import tag
 from . import Widget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
@@ -33,7 +33,8 @@ class NavButtons1(Widget):
         li_class: The class of the li tag
         ul_class: The class of the ul tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
         self[0][0][0] = ''
@@ -42,32 +43,29 @@ class NavButtons1(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        button_class = self.get_field_value('button_class')
-        li_class = self.get_field_value('li_class')
+        button_class = self.wf.button_class
+        li_class = self.wf.li_class
 
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if self.get_field_value('ul_class'):
-            self[1].update_attribs({"class":self.get_field_value('ul_class')})
+            del self[0].attribs["style"]
+        self[1].set_class_style(self.wf.ul_class)
 
         # for each link in the nav_links table - create a link and add it
         # as a list item
-        for row in self.get_field_value('nav_links'):
+        for row in self.wf.nav_links:
             linkurl, linktext, link_force_ident, link_getdata = row
             if not (linkurl or linktext):
                 continue
-            url = skiboot.get_url(linkurl, proj_ident=page.proj_ident)
+            url = self.get_url(linkurl)
             if not url:
                 continue
             lnk = tag.Part(tag_name="a", text=linktext, attribs={"role":"button"})
-            if button_class:
-                lnk.update_attribs({"class":button_class})
+            lnk.set_class_style(button_class)
             # create a url for the href
             get_field = {self.get_formname("nav_links"):link_getdata}
             url = self.make_get_url(page, url, get_field, link_force_ident)
-            lnk.update_attribs({"href": url})
+            lnk.attribs["href"] = url
             if li_class:
                 listtag = tag.Part(tag_name="li", attribs={'class':li_class})
             else:
@@ -119,34 +117,32 @@ class NavButtons2(Widget):
           3 : The get field data to send with the link
         button_class: The class of the button links - which provides the appearance via CSS
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # as this widget does not display errors, and is not json settable, hide if empty
         self.hide_if_empty=True
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the list of links"
-        button_class = self.get_field_value('button_class')
-        button_style = self.get_field_value('button_style')
+        button_class = self.wf.button_class
+        button_style = self.wf.button_style
 
         # for each link in the nav_links table - create a link and add it
         # as a list item
-        for row in self.get_field_value('nav_links'):
+        for row in self.wf.nav_links:
             linkurl, linktext, link_force_ident, link_getdata = row
             if not (linkurl or linktext):
                 continue
-            url = skiboot.get_url(linkurl, proj_ident=page.proj_ident)
+            url = self.get_url(linkurl)
             if not url:
                 continue
             lnk = tag.Part(tag_name="a", text=linktext, attribs={"role":"button"})
-            if button_class:
-                lnk.update_attribs({"class":button_class})
-            if button_style:
-                lnk.update_attribs({"style":button_style})
+            lnk.set_class_style(button_class, button_style)
             # create a url for the href
             get_field = {self.get_formname("nav_links"):link_getdata}
             url = self.make_get_url(page, url, get_field, link_force_ident)
-            lnk.update_attribs({"href": url})
+            lnk.attribs["href"] = url
             self.append(lnk)
 
     @classmethod
@@ -188,57 +184,56 @@ class NavButtons3(Widget):
         button_classes: list of CSS classes, each list item set to button links
         button_style: A style set on every button
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
-        self._jsonurl = ''
-
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the list of links"
-        button_classes = self.get_field_value('button_classes')
-        button_style = self.get_field_value('button_style')
-        button_text = self.get_field_value('button_text')
-        get_field1 = self.get_field_value('get_field1')
-        get_field2 = self.get_field_value('get_field2')
+        button_classes = self.wf.button_classes
+        button_style = self.wf.button_style
+        button_text = self.wf.button_text
+        get_field1 = self.wf.get_field1
+        get_field2 = self.wf.get_field2
 
         if not button_text:
             return
 
-        link_ident = self.get_field_value("link_ident")
+        link_ident = self.wf.link_ident
         if not link_ident:
             link_ident = 'no_javascript'
         # get url
-        linkurl = skiboot.get_url(link_ident, proj_ident=page.proj_ident)
+        linkurl = self.get_url(link_ident)
 
-        json_ident = self.get_field_value("json_ident")
+        json_ident = self.wf.json_ident
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
         if json_ident:
-            self._jsonurl = skiboot.get_url(json_ident, proj_ident=page.proj_ident)
+            self.jlabels['jsonurl'] = self.get_url(json_ident)
 
         # for each link - create a link and add it
         for index,txt in enumerate(button_text):
             lnk = tag.Part(tag_name="a", text=txt, attribs={"role":"button"})
             if button_style:
-                lnk.update_attribs({"style":button_style})
+                lnk.attribs["style"] = button_style
             if button_classes:
                 button_class = button_classes[index] if index < len(button_classes) else ''
                 if button_class:
-                    lnk.update_attribs({"class":button_class})
+                    lnk.attribs["class"] = button_class
 
             get1 = get_field1[index] if index < len(get_field1) else ''
             get2 = get_field2[index] if index < len(get_field2) else ''
             get_fields = {self.get_formname("get_field1"):get1, self.get_formname("get_field2"):get2}
             url = self.make_get_url(page, linkurl, get_fields, True)
-            lnk.update_attribs({"href": url})
+            lnk.attribs["href"] = url
             self.append(lnk)
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a click event handler"""
-        jscript = """  $("#{ident} a").click(function (e) {{
+        ident = self.get_id()
+        return f"""  $("#{ident} a").click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident = self.get_id())
-        if self._jsonurl:
-            return jscript + self._make_fieldvalues(jsonurl=self._jsonurl)
-
+"""
 
     @classmethod
     def description(cls):
@@ -254,11 +249,6 @@ class NavButtons3(Widget):
 
 
 
-
-
-
-
-
 class TabButtons1(Widget):
     """Defines a div, containing tab buttons which can hide/display portions of the page.
        Hides all elements of the page with a given class, then displays a portion of the
@@ -289,84 +279,68 @@ class TabButtons1(Widget):
         onclick_removeclass: CSS class to remove from a button when it is clicked
         active_button: the button index (starting at 0) of the button which is currently active
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self.hide_if_empty=True
-        self._display_id_list = []
-        self._hide_class = ''
-        self._onclick_addclass = ''
-        self._onclick_removeclass = ''
-        self._active = 0
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the list of buttons"
 
         # get parameters to send to javascript
-        self._hide_class = self.get_field_value('hide_class')
-        self._onclick_addclass = self.get_field_value('onclick_addclass')
-        self._onclick_removeclass = self.get_field_value('onclick_removeclass')
+        onclick_addclass = self.wf.onclick_addclass
+        onclick_removeclass = self.wf.onclick_removeclass
+
+        button_class = self.wf.button_class
+        button_style = self.wf.button_style
+
+        display_id_list = []
 
-        button_class = self.get_field_value('button_class')
-        button_style = self.get_field_value('button_style')
         # for each button in the tabs table - create a button and add it
-
-        self._active = self.get_field_value('active_button')
-
-        for index, btn in enumerate(self.get_field_value('tabs')):
+        for index, btn in enumerate(self.wf.tabs):
             buttontext, displayid = btn
             if not (buttontext or displayid):
                 continue
             btn = tag.Part(tag_name="button", text=buttontext)
-            if index == self._active:
+            btn.set_class_style(button_class, button_style)
+            if index == self.wf.active_button:
                 # this button flagged as chosen
-                if button_class and self._onclick_addclass:
-                    if button_class == self._onclick_addclass:
-                        btn.update_attribs({"class":button_class})
+                if onclick_addclass:
+                    if button_class:
+                        if button_class != onclick_addclass:
+                            btn.attribs["class"] = button_class + " " + onclick_addclass
                     else:
-                        btn.update_attribs({"class":button_class + " " + self._onclick_addclass})
-                elif button_class:
-                    btn.update_attribs({"class":button_class})
-                elif self._onclick_addclass:
-                    btn.update_attribs({"class":self._onclick_addclass})
+                        btn.attribs["class"] = onclick_addclass
             else:
-                # give each button button_class and self._onclick_removeclass which will be removed when the button is clicked
-                if button_class and self._onclick_removeclass:
-                    if button_class == self._onclick_removeclass:
-                        btn.update_attribs({"class":button_class})
+                # give each button button_class and onclick_removeclass which will be removed when the button is clicked
+                if onclick_removeclass:
+                    if button_class:
+                        if button_class != onclick_removeclass:
+                            btn.attribs["class"] = button_class + " " + onclick_removeclass
                     else:
-                        btn.update_attribs({"class":button_class + " " + self._onclick_removeclass})
-                elif button_class:
-                    btn.update_attribs({"class":button_class})
-                elif self._onclick_removeclass:
-                    btn.update_attribs({"class":self._onclick_removeclass})
-            if button_style:
-                btn.update_attribs({"style":button_style})
+                        btn.attribs["class"] = onclick_removeclass
             self.append(btn)
-            self._display_id_list.append(displayid)
+            display_id_list.append(displayid)
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if display_id_list:
+            self.jlabels['display_id_list'] = display_id_list
+        if self.wf.hide_class:
+            self.jlabels['hide_class'] = self.wf.hide_class
+        if onclick_addclass:
+            self.jlabels['onclick_addclass'] = onclick_addclass
+        if onclick_removeclass:
+            self.jlabels['onclick_removeclass'] = onclick_removeclass
 
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a click event handler"""
-        jscript = """  $("#{ident} button").click(function (e) {{
+        ident = self.get_id()
+        jscript = f"""  $("#{ident} button").click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident = self.get_id())
-        other_parameters = {}
-        if self._display_id_list:
-            other_parameters['display_id_list'] = self._display_id_list
-        if self._hide_class:
-            other_parameters['hide_class'] = self._hide_class
-        if self._onclick_addclass:
-            other_parameters['onclick_addclass'] = self._onclick_addclass
-        if self._onclick_removeclass:
-            other_parameters['onclick_removeclass'] = self._onclick_removeclass
-        if other_parameters:
-            fieldvalues = self._make_fieldvalues(**other_parameters)
-        else:
-            fieldvalues = ""
-        return jscript + fieldvalues + """  SKIPOLE.widgets["{ident}"].setbutton({active});
-""".format(ident = self.get_id(), active=self._active)
-
+  SKIPOLE.widgets["{ident}"].setbutton({self.wf.active_button});
+"""
 
     @classmethod
     def description(cls):
@@ -411,46 +385,36 @@ class DropDownButton1(Widget):
         div_class: CSS class of inner div holding links
         link_class: CSS class of each link
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self.hide_if_empty=True
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        "Build the list of links"
-        button_class = self.get_field_value('button_class')
-        button_style = self.get_field_value('button_style')
-        div_class = self.get_field_value('div_class')
-        link_class = self.get_field_value('link_class')
-
-        self[0] = tag.Part(tag_name="button", text=self.get_field_value('button_text'))
-        if button_class:
-            self[0].update_attribs({"class":button_class})
-        if button_style:
-            self[0].update_attribs({"style":button_style})
-
-        if div_class:
-            self[1] =  tag.Part(tag_name="div", attribs={"class":div_class})
-        else:
-            self[1] =  tag.Part(tag_name="div")
-
+        "Build the list of links" 
+        self[0] = tag.Part(tag_name="button", text=self.wf.button_text)
+        self[0].set_class_style(self.wf.button_class, self.wf.button_style)
+        self[1] = tag.Part(tag_name="div")
+        self[1].set_class_style(self.wf.div_class)
+        link_class = self.wf.link_class
         # for each link in the nav_links table - create a link and add it
         # as a list item
-        for row in self.get_field_value('nav_links'):
+        for row in self.wf.nav_links:
             linkurl, linktext, link_force_ident, link_getdata = row
             if not (linkurl or linktext):
                 continue
-            url = skiboot.get_url(linkurl, proj_ident=page.proj_ident)
+            url = self.get_url(linkurl)
             if not url:
                 continue
             lnk = tag.Part(tag_name="a", text=linktext)
-            if link_class:
-                lnk.update_attribs({"class":link_class})
+            lnk.set_class_style(link_class)
             # create a url for the href
             get_field = {self.get_formname("nav_links"):link_getdata}
             url = self.make_get_url(page, url, get_field, link_force_ident)
-            lnk.update_attribs({"href": url})
+            lnk.attribs["href"] = url
             self[1].append(lnk)
 
+
     @classmethod
     def description(cls):
         """Returns a text string to illustrate the widget"""
@@ -482,12 +446,13 @@ class HeaderErrorPara(Widget):
         hide: If True, sets display: none; on the widget, can be set/unset via JSON file
                     on error the value is overridden and the widget is shown
         """
-        Widget.__init__(self, name=name, tag_name="p", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "p"
         self[0] = self.error_message
 
     def _build(self, page, ident_list, environ, call_data, lang):
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
+        self.widget_hide(self.wf.hide)
 
 
     @classmethod
@@ -515,12 +480,13 @@ class HeadText(Widget):
         large_text: The text appearing in the tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="h1", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "h1"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self.tag_name = self.get_field_value('tag')
-        self[0] = self.get_field_value("large_text")
+        self.tag_name = self.wf.tag
+        self[0] = self.wf.large_text
 
     @classmethod
     def description(cls):
@@ -557,7 +523,8 @@ class HeaderText1(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h1")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -567,22 +534,13 @@ class HeaderText1(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
-
-
+            del self[2].attribs["style"]
 
     @classmethod
     def description(cls):
@@ -623,7 +581,8 @@ class HeaderText2(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h2")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -634,22 +593,13 @@ class HeaderText2(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
-
-
+            del self[2].attribs["style"]
 
     @classmethod
     def description(cls):
@@ -690,7 +640,8 @@ class HeaderText3(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h3")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -701,21 +652,13 @@ class HeaderText3(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
-
+            del self[2].attribs["style"]
 
 
     @classmethod
@@ -757,7 +700,8 @@ class HeaderText4(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h4")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -768,20 +712,13 @@ class HeaderText4(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
+            del self[2].attribs["style"]
 
 
 
@@ -824,7 +761,8 @@ class HeaderText5(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h5")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -835,21 +773,13 @@ class HeaderText5(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
-
+            del self[2].attribs["style"]
 
 
     @classmethod
@@ -891,7 +821,8 @@ class HeaderText6(Widget):
         h_class: CSS class applied to the h tag
         p_class: CSS class applied to the p tag
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name="h6")
         self[1] = tag.Part(tag_name="p")
         # error div at 2
@@ -902,20 +833,13 @@ class HeaderText6(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the header"
-        self[0].text=self.get_field_value("large_text")
-        self[1].text=self.get_field_value("small_text")
-        if self.get_field_value("h_class"):
-            self[0].update_attribs({"class":self.get_field_value("h_class")})
-        if self.get_field_value("p_class"):
-            self[1].update_attribs({"class":self.get_field_value("p_class")})
-        if self.get_field_value("h_style"):
-            self[0].update_attribs({"style":self.get_field_value("h_style")})
-        if self.get_field_value("p_style"):
-            self[1].update_attribs({"style":self.get_field_value("p_style")})
-        if self.get_field_value('error_class'):
-            self[2].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].text=self.wf.large_text
+        self[1].text=self.wf.small_text
+        self[0].set_class_style(self.wf.h_class, self.wf.h_style)
+        self[1].set_class_style(self.wf.p_class, self.wf.p_style)
+        self[2].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[2].del_one_attrib("style")
+            del self[2].attribs["style"]
 
     @classmethod
     def description(cls):
