@@ -4,7 +4,7 @@
       further input fields.  The module also has an Hidden Field and Submit Button widgets, which can be inserted into
      a form. """
 
-from .. import skiboot, tag
+from .. import tag
 from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -18,17 +18,17 @@ class HiddenField(ClosedWidget):
 
     def __init__(self, name=None, brief='', **field_args):
         "hidden_field: A hidden input field"
-        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "input"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Sets the attributes"
-        value = self.get_field_value('hidden_field')
-        if not value:
+        if not self.wf.hidden_field:
             self.show = False
             return
-        self.update_attribs({"name":self.get_formname('hidden_field'),
-                       "value":value,
-                       "type":"hidden"})
+        self.attribs.update({"name":self.get_formname('hidden_field'),
+                             "value":self.wf.hidden_field,
+                             "type":"hidden"})
 
     @classmethod
     def description(cls):
@@ -48,27 +48,28 @@ class HiddenSessionStorage(ClosedWidget):
 
     def __init__(self, name=None, brief='', **field_args):
         "hidden_field: A hidden input field with value from session storage"
-        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
-        self._key = ''
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "input"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Sets the attributes"
-        self._key = self.get_field_value('session_key')
-        if not self._key:
+        if not self.wf.session_key:
             self.show = False
             return
-        self.update_attribs({"name":self.get_formname('session_key'),
-                       "value":"",
-                       "type":"hidden"})
+        self.attribs.update({"name":self.get_formname('session_key'),
+                             "value":"",
+                             "type":"hidden"})
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['session_key'] = self.wf.session_key
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets key value into the value attribute by calling the widget updatefunc"""
-        if not self._key:
+        if not self.wf.session_key:
             return
-        jscript =  """
+        ident = self.get_id()
+        return f"""
   SKIPOLE.widgets["{ident}"].updatefunc();
-""".format(ident=self.get_id())
-        return self._make_fieldvalues(session_key=self._key) + jscript
+"""
 
     @classmethod
     def description(cls):
@@ -89,29 +90,28 @@ class HiddenLocalStorage(ClosedWidget):
 
     def __init__(self, name=None, brief='', **field_args):
         "hidden_field: A hidden input field with value from local storage"
-        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
-        self._key = ''
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "input"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Sets the attributes"
-        self._key = self.get_field_value('local_key')
-        if not self._key:
+        if not self.wf.local_key:
             self.show = False
             return
-        self.update_attribs({"name":self.get_formname('local_key'),
-                       "value":"",
-                       "type":"hidden"})
+        self.attribs.update({"name":self.get_formname('local_key'),
+                             "value":"",
+                             "type":"hidden"})
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['local_key'] = self.wf.local_key
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets key value into the value attribute by calling the widget updatefunc"""
-        if not self._key:
+        if not self.wf.local_key:
             return
-        jscript = """
+        ident = self.get_id()
+        return f"""
   SKIPOLE.widgets["{ident}"].updatefunc();
-""".format(ident=self.get_id())
-        return self._make_fieldvalues(local_key=self._key) + jscript
-
-
+"""
 
     @classmethod
     def description(cls):
@@ -132,16 +132,17 @@ class SubmitButton1(ClosedWidget):
 
     def __init__(self, name=None, brief='', **field_args):
         "Create input type submit button widget"
-        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "input"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Sets the attributes"
-        button_text = self.get_field_value('button_text')
+        button_text = self.wf.button_text
         if not button_text:
             button_text = "Submit"
-        self.update_attribs({"name":self.get_formname('button_text'),
-                       "value":button_text,
-                       "type":"submit"})
+        self.attribs.update({"name":self.get_formname('button_text'),
+                             "value":button_text,
+                             "type":"submit"})
 
     @classmethod
     def description(cls):
@@ -161,14 +162,15 @@ class SubmitButton2(ClosedWidget):
 
     def __init__(self, name=None, brief='', **field_args):
         "Create input type submit button widget"
-        ClosedWidget.__init__(self, name=name, tag_name="input", brief=brief, **field_args)
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "input"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Sets the attributes"
-        button_text = self.get_field_value('button_text')
+        button_text = self.wf.button_text
         if not button_text:
             button_text = "Submit"
-        self.update_attribs({"value":button_text, "type":"submit"})
+        self.attribs.update({"value":button_text, "type":"submit"})
 
     @classmethod
     def description(cls):
@@ -183,7 +185,7 @@ class Form1(Widget):
        On error - the error message is displayed before any of the contents
        Does not include a submit button, therefore requires one to be inserted with the contents"""
 
-    _container = ((1,0),)
+    _container =  ((1,0),)   ### ((1,0),(2,)) #########
 
     error_location = (0,0,0)
 
@@ -207,8 +209,9 @@ class Form1(Widget):
         hidden_field4: A fourth hidden field value, leave blank if unused
         container_class: the class attribute of the div holding the container
         error_class: The class applied to the paragraph containing the error message on error."""
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
-        self.update_attribs({"role":"form", "method":"post"})
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
+        self.attribs.update({"role":"form", "method":"post"})
         # error div at 0
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -218,30 +221,33 @@ class Form1(Widget):
         # The location 1,0 is available as a container
         self[1][0] = tag.Part(tag_name="div")
         self[1][0][0] = ''
+        ######################################
+        #self[2] = tag.Part(tag_name="div")
+        #self[2][0] = ''
+
+
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
-        if self.get_field_value('enctype'):
-            self[1].update_attribs({"enctype": self.get_field_value('enctype')})
+        self[1].attribs["action"] = actionurl
+        if self.wf.enctype:
+            self[1].attribs["enctype"] = self.wf.enctype
 
         # the div holding the container
-        if self.get_field_value('container_class'):
-            self[1][0].attribs = {"class": self.get_field_value('container_class')}
+        self[1][0].set_class_style(self.wf.container_class)
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -251,11 +257,11 @@ class Form1(Widget):
         """Sets a submit event handler"""
         # this ensures any input text widgets added to the container, get local validation
         # when the form is submitted
-        jscript = """  $('#{ident} form').on("submit", function(e) {{
+        ident=self.get_id()
+        return f"""  $('#{ident} form').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        return jscript
+"""
 
     @classmethod
     def description(cls):
@@ -324,7 +330,8 @@ class SubmitForm1(Widget):
         container_class: the class attribute of the div holding the container
         error_class: The class applied to the paragraph containing the error message on error.
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # error div at 0
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -339,66 +346,63 @@ class SubmitForm1(Widget):
         # the left label
         self[1][1][0] = tag.Part(tag_name="label", hide_if_empty=True)
         # the submit button
-        self[1][1][1] = tag.ClosedPart(tag_name="input")
+        self[1][1][1] = tag.ClosedPart(tag_name="input", attribs={"type":"submit"})
         # the right label
         self[1][1][2] = tag.Part(tag_name="label", hide_if_empty=True)
-        self._jsonurl = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
-        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        jsonurl = self.get_url(self.wf.action_json)
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if jsonurl:
+            self.jlabels['url'] = jsonurl
+        if self.wf.button_wait_text:
+           self.jlabels['button_wait_text'] = self.wf.button_wait_text
+
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
-        if self.get_field_value('enctype'):
-            self[1].update_attribs({"enctype": self.get_field_value('enctype')})
+        self[1].attribs["action"] = actionurl
+        if self.wf.enctype:
+            self[1].attribs["enctype"] = self.wf.enctype
 
         # the div holding the container
-        if self.get_field_value('container_class'):
-            self[1][0].attribs = {"class": self.get_field_value('container_class')}
+        self[1][0].set_class_style(self.wf.container_class)
 
         # the div holding label and button
-        if self.get_field_value('div_class'):
-            self[1][1].attribs = {"class": self.get_field_value('div_class')}
+        self[1][1].set_class_style(self.wf.div_class)
 
-        if self.get_field_value('left_label'):
-            self[1][1][0][0] = self.get_field_value('left_label')
-        if self.get_field_value('left_class'):
-            self[1][1][0].attribs = {"class": self.get_field_value('left_class')}
-        if self.get_field_value('left_style'):
-            self[1][1][0].attribs = {"style": self.get_field_value('left_style')}
+        self[1][1][0].set_class_style(self.wf.left_class, self.wf.left_style)
+        if self.wf.left_label:
+            self[1][1][0][0] = self.wf.left_label
 
         # submit button
-        if self.get_field_value('button_class'):
-            self[1][1][1].attribs = {"value":self.get_field_value('button_text'), "type":"submit", "class": self.get_field_value('button_class')}
-        else:
-            self[1][1][1].attribs = {"value":self.get_field_value('button_text'), "type":"submit"}
+        self[1][1][1].set_class_style(self.wf.button_class)
+        self[1][1][1].attribs["value"] = self.wf.button_text
 
         # set an id in the submit button for the 'label for' tag
-        self[1][1][1].insert_id()
+        for_id = self[1][1][1].insert_id()
 
-        if self.get_field_value('right_label'):
-            self[1][1][2][0] = self.get_field_value('right_label')
-        if self.get_field_value('right_class'):
-            self[1][1][2].attribs = {"class": self.get_field_value('right_class')}
-        if self.get_field_value('right_style'):
-            self[1][1][2].attribs = {"style": self.get_field_value('right_style')}
+        self[1][1][2].set_class_style(self.wf.right_class, self.wf.right_style)
+        if self.wf.right_label:
+            self[1][1][2][0] = self.wf.right_label
 
         # set the label 'for' attribute
-        self[1][1][0].update_attribs({'for':self[1][1][1].get_id()})
-        self[1][1][2].update_attribs({'for':self[1][1][1].get_id()})
+        self[1][1][0].attribs['for'] = for_id
+        self[1][1][2].attribs['for'] = for_id
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['buttonident'] = for_id
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -406,15 +410,11 @@ class SubmitForm1(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
-        jscript = """$('#{ident} form').on("submit", function(e) {{
+        ident=self.get_id()
+        return f"""$('#{ident} form').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        if self._jsonurl:
-            return jscript + self._make_fieldvalues('button_wait_text', buttonident = self[1][1][1].get_id(), url=self._jsonurl)
-        else:
-            return jscript + self._make_fieldvalues('button_wait_text', buttonident = self[1][1][1].get_id())
-
+"""
 
     @classmethod
     def description(cls):
@@ -496,7 +496,8 @@ class SubmitForm2(Widget):
         container_class: the class attribute of the div holding the container
         error_class: The class applied to the paragraph containing the error message on error.
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # error div at 0
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -511,66 +512,68 @@ class SubmitForm2(Widget):
         # the left label
         self[1][1][0] = tag.Part(tag_name="label", hide_if_empty=True)
         # the submit button
-        self[1][1][1] = tag.ClosedPart(tag_name="input")
+        self[1][1][1] = tag.ClosedPart(tag_name="input", attribs={"type":"submit"})
         # the right label
         self[1][1][2] = tag.Part(tag_name="label", hide_if_empty=True)
-        self._jsonurl = ''
+
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
-        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        jsonurl = self.get_url(self.wf.action_json)
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if jsonurl:
+            self.jlabels['url'] = jsonurl
+        if self.wf.button_wait_text:
+           self.jlabels['button_wait_text'] = self.wf.button_wait_text
+        if self.wf.session_storage:
+           self.jlabels['session_storage'] = self.wf.session_storage
+        if self.wf.local_storage:
+           self.jlabels['local_storage'] = self.wf.local_storage
+
+        self[0].set_class_style(self.wf.error_class)
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
-        if self.get_field_value('enctype'):
-            self[1].update_attribs({"enctype": self.get_field_value('enctype')})
+        self[1].attribs["action"] = actionurl
+        if self.wf.enctype:
+            self[1].attribs["enctype"] = self.wf.enctype
 
         # the div holding the container
-        if self.get_field_value('container_class'):
-            self[1][0].attribs = {"class": self.get_field_value('container_class')}
+        self[1][0].set_class_style(self.wf.container_class)
 
         # the div holding label and button
-        if self.get_field_value('div_class'):
-            self[1][1].attribs = {"class": self.get_field_value('div_class')}
+        self[1][1].set_class_style(self.wf.div_class)
 
-        if self.get_field_value('left_label'):
-            self[1][1][0][0] = self.get_field_value('left_label')
-        if self.get_field_value('left_class'):
-            self[1][1][0].attribs = {"class": self.get_field_value('left_class')}
-        if self.get_field_value('left_style'):
-            self[1][1][0].attribs = {"style": self.get_field_value('left_style')}
+        self[1][1][0].set_class_style(self.wf.left_class, self.wf.left_style)
+        if self.wf.left_label:
+            self[1][1][0][0] = self.wf.left_label
 
         # submit button
-        if self.get_field_value('button_class'):
-            self[1][1][1].attribs = {"value":self.get_field_value('button_text'), "type":"submit", "class": self.get_field_value('button_class')}
-        else:
-            self[1][1][1].attribs = {"value":self.get_field_value('button_text'), "type":"submit"}
+        self[1][1][1].set_class_style(self.wf.button_class)
+        self[1][1][1].attribs["value"] = self.wf.button_text
 
         # set an id in the submit button for the 'label for' tag
-        self[1][1][1].insert_id()
+        for_id = self[1][1][1].insert_id()
 
-        if self.get_field_value('right_label'):
-            self[1][1][2][0] = self.get_field_value('right_label')
-        if self.get_field_value('right_class'):
-            self[1][1][2].attribs = {"class": self.get_field_value('right_class')}
-        if self.get_field_value('right_style'):
-            self[1][1][2].attribs = {"style": self.get_field_value('right_style')}
+        self[1][1][2].set_class_style(self.wf.right_class, self.wf.right_style)
+        if self.wf.right_label:
+            self[1][1][2][0] = self.wf.right_label
 
         # set the label 'for' attribute
-        self[1][1][0].update_attribs({'for':self[1][1][1].get_id()})
-        self[1][1][2].update_attribs({'for':self[1][1][1].get_id()})
+        self[1][1][0].attribs['for'] = for_id
+        self[1][1][2].attribs['for'] = for_id
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['buttonident'] = for_id
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -578,22 +581,11 @@ class SubmitForm2(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
-        jscript = """$('#{ident} form').on("submit", function(e) {{
+        ident=self.get_id()
+        return f"""$('#{ident} form').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        if self._jsonurl:
-            return jscript + self._make_fieldvalues('button_wait_text',
-                                                    'session_storage',
-                                                    'local_storage',
-                                                    buttonident = self[1][1][1].get_id(),
-                                                    url=self._jsonurl)
-        else:
-            return jscript + self._make_fieldvalues('button_wait_text',
-                                                    'session_storage',
-                                                    'local_storage',
-                                                    buttonident = self[1][1][1].get_id())
-
+"""
 
     @classmethod
     def description(cls):
@@ -655,77 +647,83 @@ class SubmitFromScript(Widget):
         buttondiv_style: the style attribute of the div which contains the submit button
         hide: If True, widget is hidden
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
-
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # The form
         self[0] = tag.Part(tag_name='form', attribs={"role":"form", "method":"post"})
-
         # div containing the submit button
         self[0][0] = tag.Part(tag_name='div')
         # the submit button
         self[0][0][0] = tag.Part(tag_name="button", attribs ={"type":"submit"})
         self[0][0][0][0] = "Submit"
-        self._jsonurl = ''
+
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
-        if self.get_field_value("target"):
-            self[0].update_attribs({"target":self.get_field_value("target")})
+        if self.wf.target:
+            self[0].attribs["target"] = self.wf.target
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        self._jsonurl = skiboot.get_url(self.get_field_value("action_json"), proj_ident=page.proj_ident)
+        self.widget_hide(self.wf.hide)
 
-        if not self.get_field_value("action"):
+        jsonurl = self.get_url(self.wf.action_json)
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if jsonurl:
+            self.jlabels['url'] = jsonurl
+        if self.wf.hidden_field1:
+           self.jlabels['hidden_field1'] = self.wf.hidden_field1
+        if self.wf.hidden_field2:
+           self.jlabels['hidden_field2'] = self.wf.hidden_field2
+        if self.wf.local_storage:
+           self.jlabels['hidden_field3'] = self.wf.hidden_field3
+        if self.wf.local_storage:
+           self.jlabels['hidden_field4'] = self.wf.hidden_field4
+
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[0].update_attribs({"action": actionurl})
+        self[0].attribs["action"] = actionurl
 
         # the div holding the submit button
-        if self.get_field_value('buttondiv_class'):
-            self[0][0].attribs = {"class": self.get_field_value('buttondiv_class')}
-        if self.get_field_value('buttondiv_style'):
-            self[0][0].update_attribs({"style": self.get_field_value('buttondiv_style')})
+        self[0][0].set_class_style(self.wf.buttondiv_class, self.wf.buttondiv_style)
 
         # submit button
-        if self.get_field_value('button_class'):
-            self[0][0][0].update_attribs({"class": self.get_field_value('button_class')})
-        if self.get_field_value('button_text'):
-            self[0][0][0][0] = self.get_field_value('button_text')
+        self[0][0][0].set_class_style(self.wf.button_class)
+        if self.wf.button_text:
+            self[0][0][0][0] = self.wf.button_text
 
         # add ident and four hidden fields
-
         if page is not None:
             self[0].append(tag.ClosedPart(tag_name="input",
                                    attribs ={"name":'ident',
                                              "value":page.ident_data_string,
                                              "type":"hidden"}))
         # hidden field on the form
-        if self.get_field_value('hidden_field1'):
+        if self.wf.hidden_field1:
             self[0].append(tag.ClosedPart(tag_name="input",
                                        attribs ={"name":self.get_formname('hidden_field1'),
                                                  "type":"hidden"}))
 
         # Second hidden field on the form
-        if self.get_field_value('hidden_field2'):
+        if self.wf.hidden_field2:
             self[0].append(tag.ClosedPart(tag_name="input",
                                        attribs ={"name":self.get_formname('hidden_field2'),
                                                  "type":"hidden"}))
 
         # third hidden field on the form
-        if self.get_field_value('hidden_field3'):
+        if self.wf.hidden_field3:
             self[0].append(tag.ClosedPart(tag_name="input",
                                        attribs ={"name":self.get_formname('hidden_field3'),
                                                  "type":"hidden"}))
 
         # fourth hidden field on the form
-        if self.get_field_value('hidden_field4'):
+        if self.wf.hidden_field4:
             self[0].append(tag.ClosedPart(tag_name="input",
                                        attribs ={"name":self.get_formname('hidden_field4'),
                                                  "type":"hidden"}))
@@ -733,13 +731,11 @@ class SubmitFromScript(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
-        jscript = """  $("#{ident} form").on("submit input", function(e) {{
+        ident=self.get_id()
+        return f"""  $("#{ident} form").on("submit input", function(e) {{
     SKIPOLE.widgets["{ident}"].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        if self._jsonurl:
-            return jscript + self._make_fieldvalues('hidden_field1', 'hidden_field2', 'hidden_field3', 'hidden_field4', url=self._jsonurl)
-        return jscript + self._make_fieldvalues('hidden_field1', 'hidden_field2', 'hidden_field3', 'hidden_field4')
+"""
 
 
     @classmethod
