@@ -1,9 +1,7 @@
 
 """Contains widgets for inputting data"""
 
-from urllib.parse import quote
-
-from .. import skiboot, tag, excepts
+from .. import tag, excepts
 from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -46,29 +44,30 @@ class InputTable1(Widget):
                    the fields submitted will have name 'widgetname:inputdict-keyname'
                    and the user will receive a widgfield ('widgetname','inputdict') containing a dictionary of keyname:values
          """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        inputdict = self.get_field_value("inputdict")
-        rowc = self.get_field_value("row_classes")
-        col1 = self.get_field_value("col1")
-        col2 = self.get_field_value("col2")
+        inputdict = self.wf.inputdict
+        rowc = self.wf.row_classes
+        col1 = self.wf.col1
+        col2 = self.wf.col2
         input_name = self.get_formname("inputdict") + '-'
-        size = self.get_field_value('size')
-        maxlength = self.get_field_value('maxlength')
+        size = self.wf.size
+        maxlength = self.wf.maxlength
 
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2') or self.get_field_value('title3'):
+        if self.wf.title1 or self.wf.title2 or self.wf.title3:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
-            self[0][2] = tag.Part(tag_name='th', text = self.get_field_value('title3'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
+            self[0][2] = tag.Part(tag_name='th', text = self.wf.title3)
 
         # create rows
         rows = max( len(col1), len(col2), len(inputdict) )
@@ -101,9 +100,9 @@ class InputTable1(Widget):
                 keyed_name = input_name + key
                 self[rownumber][2][0] = tag.ClosedPart(tag_name="input", attribs={"name":keyed_name, "type":"text", "value":inputdict[key]})
                 if size:
-                    self[rownumber][2][0].update_attribs({"size":size})
+                    self[rownumber][2][0].attribs["size"] = size
                 if maxlength:
-                    self[rownumber][2][0].update_attribs({"maxlength":maxlength})
+                    self[rownumber][2][0].attribs["maxlength"] = maxlength
 
 
 
@@ -191,7 +190,8 @@ class InputTable5(Widget):
         set_input_accepted: Dictionary of col index keys that should be set with input_accepted_class
         set_input_errored: Dictionary of col index keys that should be set with input_errored_class
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
@@ -201,19 +201,19 @@ class InputTable5(Widget):
         ident_value = page.ident_data_string
 
         # get the form action url
-        action_url = skiboot.get_url(self.get_field_value("action"), proj_ident=page.proj_ident)
+        action_url = self.get_url(self.wf.action)
         if not action_url:
             # setting self._error replaces the entire tag by the self._error message
             self._error = "Warning: broken link"
             return
 
 
-        len_label = len(self.fields["col_label"])
-        len_input = len(self.fields["col_input"])
-        len_hidden_field1 = len(self.fields["hidden_field1"])
-        len_hidden_field2 = len(self.fields["hidden_field2"])
-        len_hidden_field3 = len(self.fields["hidden_field3"])
-        len_hidden_field4 = len(self.fields["hidden_field4"])
+        len_label = len(self.wf.col_label)
+        len_input = len(self.wf.col_input)
+        len_hidden_field1 = len(self.wf.hidden_field1)
+        len_hidden_field2 = len(self.wf.hidden_field2)
+        len_hidden_field3 = len(self.wf.hidden_field3)
+        len_hidden_field4 = len(self.wf.hidden_field4)
 
         rows = max(len_label, len_input)
         if not rows:
@@ -221,58 +221,63 @@ class InputTable5(Widget):
 
         input_name = self.get_formname("col_input")
         button_name1 = self.get_formname("button_text1")
-        button_value1 = self.get_field_value('button_text1')
-        button1_class = self.get_field_value('button1_class')
+        button_value1 = self.wf.button_text1
+        button1_class = self.wf.button1_class
         button_name2 = self.get_formname("button_text2")
-        button_value2 = self.get_field_value('button_text2')
-        button2_class = self.get_field_value('button2_class')
+        button_value2 = self.wf.button_text2
+        button2_class = self.wf.button2_class
         hidden_field1_name = self.get_formname("hidden_field1")
         hidden_field2_name = self.get_formname("hidden_field2")
         hidden_field3_name = self.get_formname("hidden_field3")
         hidden_field4_name = self.get_formname("hidden_field4")
 
-        label_class = self.get_field_value('label_class')
-        label_style = self.get_field_value('label_style')
+        label_class = self.wf.label_class
+        label_style = self.wf.label_style
 
         form_id = self.get_id() + '_'
 
-        if self.get_field_value('input_accepted_class') and self.get_field_value("set_input_accepted"):
-            set_input_accepted = self.get_field_value("set_input_accepted")
+        if self.wf.input_accepted_class and self.wf.set_input_accepted:
+            set_input_accepted = self.wf.set_input_accepted
         else:
             set_input_accepted = {}
 
-        if self.get_field_value('input_errored_class') and self.get_field_value("set_input_errored"):
-            set_input_errored = self.get_field_value("set_input_errored")
+        if self.wf.input_errored_class and self.wf.set_input_errored:
+            set_input_errored = self.wf.set_input_errored
         else:
             set_input_errored = {}
 
-        input_class = self.get_field_value('input_class')
-        input_style = self.get_field_value('input_style')
+        input_class = self.wf.input_class
+        input_style = self.wf.input_style
 
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        if self.wf.input_accepted_class:
+            self.jlabels['input_accepted_class'] = self.wf.input_accepted_class
+        if self.wf.input_errored_class:
+            self.jlabels['input_errored_class'] = self.wf.input_errored_class
 
         for rownumber in range(rows):
             if rownumber<len_label:
-                col_label = self.get_field_value("col_label")[rownumber]
+                col_label = self.wf.col_label[rownumber]
             else:
                 col_label = ''
             if rownumber<len_input:
-                col_input = self.get_field_value("col_input")[rownumber]
+                col_input = self.wf.col_input[rownumber]
             else:
                 col_input = ''
             if rownumber<len_hidden_field1:
-                col_hidden_field1 = self.get_field_value("hidden_field1")[rownumber]
+                col_hidden_field1 = self.wf.hidden_field1[rownumber]
             else:
                 col_hidden_field1 = ''
             if rownumber<len_hidden_field2:
-                col_hidden_field2 = self.get_field_value("hidden_field2")[rownumber]
+                col_hidden_field2 = self.wf.hidden_field2[rownumber]
             else:
                 col_hidden_field2 = ''
             if rownumber<len_hidden_field3:
-                col_hidden_field3 = self.get_field_value("hidden_field3")[rownumber]
+                col_hidden_field3 = self.wf.hidden_field3[rownumber]
             else:
                 col_hidden_field3 = ''
             if rownumber<len_hidden_field4:
-                col_hidden_field4 = self.get_field_value("hidden_field4")[rownumber]
+                col_hidden_field4 = self.wf.hidden_field4[rownumber]
             else:
                 col_hidden_field4 = ''
 
@@ -285,7 +290,7 @@ class InputTable5(Widget):
             else:
                 formrow[0] = tag.Part(tag_name="label")
             if label_style:
-                formrow[0].update_attribs({"style":label_style})
+                formrow[0].attribs["style"] = label_style
             formrow[0][0] = col_label
 
             # 2nd column is a text input field
@@ -293,32 +298,31 @@ class InputTable5(Widget):
                                     attribs ={"name":input_name,
                                     "value":col_input,
                                     "type":"text"})
-            if self.get_field_value('size'):
-                formrow[1].update_attribs({"size":self.get_field_value('size')})
-            if self.get_field_value('maxlength'):
-                formrow[1].update_attribs({"maxlength":self.get_field_value('maxlength')})
-            if self.get_field_value('required'):
-                formrow[1].update_attribs({"required":"required"})
+            if self.wf.size:
+                formrow[1].attribs["size"] = self.wf.size
+            if self.wf.maxlength:
+                formrow[1].attribs["maxlength"] = self.wf.maxlength
+            if self.wf.required:
+                formrow[1].attribs["required"] = "required"
             # set an id in the input field for the 'label for' tag
             this_id = form_id + str(rownumber) + '_0_1'
-            formrow[1].insert_id(this_id)
             # set the label 'for' attribute
-            formrow[0].update_attribs({'for':formrow[1].get_id()})
+            formrow[0].attribs['for'] = formrow[1].insert_id(this_id)
             if (rownumber in set_input_errored) and set_input_errored[rownumber]:
                 if input_class:
-                    formrow[1].update_attribs({"class":input_class + ' ' + self.get_field_value('input_errored_class')})
+                    formrow[1].attribs["class"] = input_class + ' ' + self.wf.input_errored_class
                 else:
-                    formrow[1].update_attribs({"class":self.get_field_value('input_errored_class')})
+                    formrow[1].attribs["class"] = self.wf.input_errored_class
             elif (rownumber in set_input_accepted) and set_input_accepted[rownumber]:
                 if input_class:
-                    formrow[1].update_attribs({"class":input_class + ' ' + self.get_field_value('input_accepted_class')})
+                    formrow[1].attribs["class"] = input_class + ' ' + self.wf.input_accepted_class
                 else:
-                    formrow[1].update_attribs({"class":self.get_field_value('input_accepted_class')})
+                    formrow[1].attribs["class"] = self.wf.input_accepted_class
             elif input_class:
-                formrow[1].update_attribs({"class":input_class})
+                formrow[1].attribs["class"] = input_class
 
             if input_style:
-                formrow[1].update_attribs({"style":input_style})
+                formrow[1].attribs["style"] = input_style
 
             # 3rd column is a submit button
             if button_value1:
@@ -369,20 +373,19 @@ class InputTable5(Widget):
 
             # place form in a div
             self[rownumber] = tag.Part(tag_name="div")
-            if self.get_field_value('div_class'):
-                self[rownumber].attribs = {"class": self.get_field_value('div_class')}
+            self[rownumber].set_class_style(self.wf.div_class)
             self[rownumber][0] = formrow
 
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a submit event handler"""
-        jscript = """  $('#{ident}').find( 'form').each(function() {{
+        ident=self.get_id()
+        return f"""  $('#{ident}').find('form').each(function() {{
     $(this).on("submit", function(e) {{
         SKIPOLE.widgets['{ident}'].eventfunc(e);
         }});
     }});
-""".format(ident=self.get_id())
-        return jscript + self._make_fieldvalues('input_accepted_class', 'input_errored_class')
+"""
 
     @classmethod
     def description(cls):
@@ -485,76 +488,75 @@ class InputTable4(Widget):
                    the fields submitted will have name 'widgetname:inputdict-keyname'
                    and the user will receive a widgfield ('widgetname','inputdict') containing a dictionary of keyname:values
          """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
-        self._up_jsonurl = ''
-        self._down_jsonurl = ''
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        inputdict = self.get_field_value("inputdict")
-        rowc = self.get_field_value("row_classes")
-        col1 = self.get_field_value("col1")
-        col2 = self.get_field_value("col2")
-        col3 = self.get_field_value("col3")
-        up_hide = self.get_field_value("up_hide")
-        down_hide = self.get_field_value("down_hide")
-
+        inputdict = self.wf.inputdict
+        rowc = self.wf.row_classes
+        col1 = self.wf.col1
+        col2 = self.wf.col2
+        col3 = self.wf.col3
+        up_hide = self.wf.up_hide
+        down_hide = self.wf.down_hide
 
         input_name = self.get_formname("inputdict") + '-'
 
-        col1_class = self.get_field_value("col1_class")
-        col2_class = self.get_field_value("col2_class")
-        col3_class = self.get_field_value("col3_class")
-        col4_class = self.get_field_value("col4_class")
-        col4_style = self.get_field_value("col4_style")
+        col1_class = self.wf.col1_class
+        col2_class = self.wf.col2_class
+        col3_class = self.wf.col3_class
+        col4_class = self.wf.col4_class
+        col4_style = self.wf.col4_style
 
-        up_link_ident = self.get_field_value("up_link_ident") # ident of the up arrow link if javascript disabled
+        up_link_ident = up_link_ident # ident of the up arrow link if javascript disabled
         if not up_link_ident:
             up_link_ident = 'no_javascript'
-        up_json_ident = self.get_field_value("up_json_ident") # ident of the up arrow link, expects a json file returned
+        up_json_ident = self.wf.up_json_ident # ident of the up arrow link, expects a json file returned
 
-        down_link_ident = self.get_field_value("down_link_ident") # ident of the down arrow link if javascript disabled
+        down_link_ident = self.wf.down_link_ident # ident of the down arrow link if javascript disabled
         if not down_link_ident:
             down_link_ident = 'no_javascript'
-        down_json_ident = self.get_field_value("down_json_ident") # ident of the down arrow link, expects a json file returned
+        down_json_ident = self.wf.down_json_ident # ident of the down arrow link, expects a json file returned
 
-        up_style = self.get_field_value("up_style") # CSS style applied to the up arrows
-        down_style = self.get_field_value("down_style") # CSS style applied to the down arrows
-        up_class = self.get_field_value("up_class") # CSS class applied to the up arrows
-        down_class = self.get_field_value("down_class") # CSS class applied to the down arrows
+        up_style = self.wf.up_style     # CSS style applied to the up arrows
+        down_style = self.wf.down_style # CSS style applied to the down arrows
+        up_class = self.wf.up_class     # CSS class applied to the up arrows
+        down_class = self.wf.down_class # CSS class applied to the down arrows
 
-        up_getfield1 = self.get_field_value("up_getfield1") # list of get fields, one for each up arrow link
-        up_getfield2 = self.get_field_value("up_getfield2") # list of second get fields, one for each up arrow link
-        down_getfield1 = self.get_field_value("down_getfield1") # list of get fields, one for each down arrow link
-        down_getfield2 = self.get_field_value("down_getfield2") # list of get fields, one for each down arrow link
+        up_getfield1 = self.wf.up_getfield1     # list of get fields, one for each up arrow link
+        up_getfield2 = self.wf.up_getfield2     # list of second get fields, one for each up arrow link
+        down_getfield1 = self.wf.down_getfield1 # list of get fields, one for each down arrow link
+        down_getfield2 = self.wf.down_getfield2 # list of get fields, one for each down arrow link
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
 
         # up arrow link
         # get json url
         if up_json_ident:
-            self._up_jsonurl = skiboot.get_url(up_json_ident, proj_ident=page.proj_ident)
+            self.jlabels["upurl"] = self.get_url(up_json_ident)
         # get url
-        up_url = skiboot.get_url(up_link_ident, proj_ident=page.proj_ident)
+        up_url = self.get_url(up_link_ident)
 
         # down arrow link
         # get json url
         if down_json_ident:
-            self._down_jsonurl = skiboot.get_url(down_json_ident, proj_ident=page.proj_ident)
+            self.jlabels["downurl"] = self.get_url(down_json_ident)
         # get url
-        down_url = skiboot.get_url(down_link_ident, proj_ident=page.proj_ident)
-
+        down_url = self.get_url(down_link_ident)
 
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2') or self.get_field_value('title3') or self.get_field_value('title4'):
+        if self.wf.title1 or self.wf.title2 or self.wf.title3 or self.wf.title4:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
-            self[0][2] = tag.Part(tag_name='th', text = self.get_field_value('title3'))
-            self[0][3] = tag.Part(tag_name='th', text = self.get_field_value('title4'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
+            self[0][2] = tag.Part(tag_name='th', text = self.wf.title3)
+            self[0][3] = tag.Part(tag_name='th', text = self.wf.title4)
 
         # create rows
         rows = max( len(col1), len(col2), len(col3), len(inputdict) )
@@ -605,7 +607,7 @@ class InputTable4(Widget):
                 self[rownumber][3] = tag.Part(tag_name='td')
 
             if col4_style:
-                self[rownumber][3].update_attribs({"style": col4_style})
+                self[rownumber][3].attribs["style"] = col4_style
 
             if up_style:
                 up_button_style = up_style
@@ -628,7 +630,7 @@ class InputTable4(Widget):
                 self[rownumber][3][0] = tag.Part(tag_name="a", attribs={"role":"button"})
 
             if up_class:
-                self[rownumber][3][0].update_attribs({"class": up_class})
+                self[rownumber][3][0].attribs["class"] = up_class
 
             self[rownumber][3][0][0] = tag.HTMLSymbol("&uarr;")
 
@@ -637,8 +639,7 @@ class InputTable4(Widget):
             upget2 = up_getfield2[index] if index < len(up_getfield2) else ''
             get_fields = {self.get_formname("up_getfield1"):upget1,
                           self.get_formname("up_getfield2"):upget2}
-            url = self.make_get_url(page, up_url, get_fields, True)
-            self[rownumber][3][0].update_attribs({"href": url})
+            self[rownumber][3][0].attribs["href"] = self.make_get_url(page, up_url, get_fields, True)
 
             if down_style:
                 down_button_style = down_style
@@ -660,7 +661,7 @@ class InputTable4(Widget):
                 self[rownumber][3][1] = tag.Part(tag_name="a", attribs={"role":"button"})
 
             if down_class:
-                self[rownumber][3][1].update_attribs({"class": down_class})
+                self[rownumber][3][1].attribs["class"] = down_class
 
             self[rownumber][3][1][0] = tag.HTMLSymbol("&darr;")
 
@@ -669,8 +670,7 @@ class InputTable4(Widget):
             downget2 = down_getfield2[index] if index < len(down_getfield2) else ''
             get_fields = {self.get_formname("down_getfield1"):downget1,
                           self.get_formname("down_getfield2"):downget2}
-            url = self.make_get_url(page, down_url, get_fields, True)
-            self[rownumber][3][1].update_attribs({"href": url})
+            self[rownumber][3][1].attribs["href"] = self.make_get_url(page, down_url, get_fields, True)
 
             key = keylist[index]
             if key:  # this is the dictionary key
@@ -680,14 +680,11 @@ class InputTable4(Widget):
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a click event handler"""
-        jscript = """  $("#{ident} a").click(function (e) {{
+        ident=self.get_id()
+        return f"""  $("#{ident} a").click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident = self.get_id())
-        if self._up_jsonurl or self._down_jsonurl:
-            return jscript + self._make_fieldvalues(upurl=self._up_jsonurl, downurl=self._down_jsonurl)
-
-
+"""
 
     @classmethod
     def description(cls):
@@ -814,83 +811,82 @@ class InputTable3(Widget):
                    and the user will receive a widgfield ('widgetname','inputdict') containing a dictionary of keyname:values
                    If values are None, then the values listed in getfield3 will be used, otherwise values given here have priority
          """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
-        self._up_jsonurl = ''
-        self._down_jsonurl = ''
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        inputdict = self.get_field_value("inputdict")
-        rowc = self.get_field_value("row_classes")
-        col1 = self.get_field_value("col1")
-        col2 = self.get_field_value("col2")
-        up_hide = self.get_field_value("up_hide")
-        down_hide = self.get_field_value("down_hide")
-
+        inputdict = self.wf.inputdict
+        rowc = self.wf.row_classes
+        col1 = self.wf.col1
+        col2 = self.wf.col2
+        up_hide = self.wf.up_hide
+        down_hide = self.wf.down_hide
 
         input_name = self.get_formname("inputdict") + '-'
 
-        col1_class = self.get_field_value("col1_class")
-        col2_class = self.get_field_value("col2_class")
-        col3_class = self.get_field_value("col3_class")
-        col4_class = self.get_field_value("col4_class")
-        col4_style = self.get_field_value("col4_style")
+        col1_class = self.wf.col1_class
+        col2_class = self.wf.col2_class
+        col3_class = self.wf.col3_class
+        col4_class = self.wf.col4_class
+        col4_style = self.wf.col4_style
 
-        input_class = self.get_field_value("input_class")
-        input_style = self.get_field_value("input_style")
+        input_class = self.wf.input_class
+        input_style = self.wf.input_style
 
-        size = self.get_field_value('size')
-        maxlength = self.get_field_value('maxlength')
+        size = self.wf.size
+        maxlength = self.wf.maxlength
 
-        up_link_ident = self.get_field_value("up_link_ident") # ident of the up arrow link if javascript disabled
+        up_link_ident = self.wf.up_link_ident     # ident of the up arrow link if javascript disabled
         if not up_link_ident:
             up_link_ident = 'no_javascript'
-        up_json_ident = self.get_field_value("up_json_ident") # ident of the up arrow link, expects a json file returned
+        up_json_ident = self.wf.up_json_ident     # ident of the up arrow link, expects a json file returned
 
-        down_link_ident = self.get_field_value("down_link_ident") # ident of the down arrow link if javascript disabled
+        down_link_ident = self.wf.down_link_ident # ident of the down arrow link if javascript disabled
         if not down_link_ident:
             down_link_ident = 'no_javascript'
-        down_json_ident = self.get_field_value("down_json_ident") # ident of the down arrow link, expects a json file returned
+        down_json_ident = self.wf.down_json_ident # ident of the down arrow link, expects a json file returned
 
-        up_style = self.get_field_value("up_style") # CSS style applied to the up arrows
-        down_style = self.get_field_value("down_style") # CSS style applied to the down arrows
-        up_class = self.get_field_value("up_class") # CSS class applied to the up arrows
-        down_class = self.get_field_value("down_class") # CSS class applied to the down arrows
+        up_style = self.wf.up_style               # CSS style applied to the up arrows
+        down_style = self.wf.down_style           # CSS style applied to the down arrows
+        up_class = self.wf.up_class               # CSS class applied to the up arrows
+        down_class = self.wf.down_class           # CSS class applied to the down arrows
 
-        up_getfield1 = self.get_field_value("up_getfield1") # list of get fields, one for each up arrow link
-        up_getfield2 = self.get_field_value("up_getfield2") # list of second get fields, one for each up arrow link
-        down_getfield1 = self.get_field_value("down_getfield1") # list of get fields, one for each down arrow link
-        down_getfield2 = self.get_field_value("down_getfield2") # list of get fields, one for each down arrow link
+        up_getfield1 = self.wf.up_getfield1       # list of get fields, one for each up arrow link
+        up_getfield2 = self.wf.up_getfield2       # list of second get fields, one for each up arrow link
+        down_getfield1 = self.wf.down_getfield1   # list of get fields, one for each down arrow link
+        down_getfield2 = self.wf.down_getfield2   # list of get fields, one for each down arrow link
 
-        getfield3 = self.get_field_value("getfield3") # list of get fields, one for each row, sent with both arrows
+        getfield3 = self.wf.getfield3             # list of get fields, one for each row, sent with both arrows
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
 
         # up arrow link
         # get json url
         if up_json_ident:
-            self._up_jsonurl = skiboot.get_url(up_json_ident, proj_ident=page.proj_ident)
+            self.jlabels["upurl"] = self.get_url(up_json_ident)
         # get url
-        up_url = skiboot.get_url(up_link_ident, proj_ident=page.proj_ident)
+        up_url = self.get_url(up_link_ident)
 
         # down arrow link
         # get json url
         if down_json_ident:
-            self._down_jsonurl = skiboot.get_url(down_json_ident, proj_ident=page.proj_ident)
+            self.jlabels["downurl"] = self.get_url(down_json_ident)
         # get url
-        down_url = skiboot.get_url(down_link_ident, proj_ident=page.proj_ident)
-
+        down_url = self.get_url(down_link_ident)
 
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2') or self.get_field_value('title3') or self.get_field_value('title4'):
+        if self.wf.title1 or self.wf.title2 or self.wf.title3 or self.wf.title4:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
-            self[0][2] = tag.Part(tag_name='th', text = self.get_field_value('title3'))
-            self[0][3] = tag.Part(tag_name='th', text = self.get_field_value('title4'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
+            self[0][2] = tag.Part(tag_name='th', text = self.wf.title3)
+            self[0][3] = tag.Part(tag_name='th', text = self.wf.title4)
 
         # create rows
         rows = max( len(col1), len(col2), len(inputdict) )
@@ -951,24 +947,19 @@ class InputTable3(Widget):
                 self[rownumber][2][0] = tag.ClosedPart(tag_name="input", attribs={"name":keyed_name,
                                                                                   "type":"text",
                                                                                    "value":inputdict[key],
-                "onchange":"SKIPOLE.widgets['{ident}'].setnewnumber(this.value, {data})".format(ident = self.get_id(), data=rownumber)
+                                             "onchange":f"SKIPOLE.widgets['{self.get_id()}'].setnewnumber(this.value, {rownumber})"
                                                                                  })
                 if size:
-                    self[rownumber][2][0].update_attribs({"size":size})
+                    self[rownumber][2][0].attribs["size"] = size
                 if maxlength:
-                    self[rownumber][2][0].update_attribs({"maxlength":maxlength})
+                    self[rownumber][2][0].attribs["maxlength"] = maxlength
                 if input_class:
-                    self[rownumber][2][0].update_attribs({"class":input_class})
+                    self[rownumber][2][0].attribs["class"] = input_class
                 if input_style:
-                    self[rownumber][2][0].update_attribs({"style":input_style})
+                    self[rownumber][2][0].attribs["style"] = input_style
 
-            if col4_class:
-                self[rownumber][3] = tag.Part(tag_name='td', attribs={"class":col4_class})
-            else:
-                self[rownumber][3] = tag.Part(tag_name='td')
-
-            if col4_style:
-                self[rownumber][3].update_attribs({"style": col4_style})
+            self[rownumber][3] = tag.Part(tag_name='td')
+            self[rownumber][3].set_class_style(col4_class, col4_style)
 
             if up_style:
                 up_button_style = up_style
@@ -990,7 +981,7 @@ class InputTable3(Widget):
                 self[rownumber][3][0] = tag.Part(tag_name="a", attribs={"role":"button"})
 
             if up_class:
-                self[rownumber][3][0].update_attribs({"class": up_class})
+                self[rownumber][3][0].attribs["class"] = up_class
 
             self[rownumber][3][0][0] = tag.HTMLSymbol("&uarr;")
 
@@ -1001,8 +992,7 @@ class InputTable3(Widget):
                           self.get_formname("up_getfield2"):upget2,
                           self.get_formname("getfield3"):get3
                           }
-            url = self.make_get_url(page, up_url, get_fields, True)
-            self[rownumber][3][0].update_attribs({"href": url})
+            self[rownumber][3][0].attribs["href"] = self.make_get_url(page, up_url, get_fields, True)
 
             if down_style:
                 down_button_style = down_style
@@ -1024,7 +1014,7 @@ class InputTable3(Widget):
                 self[rownumber][3][1] = tag.Part(tag_name="a", attribs={"role":"button"})
 
             if down_class:
-                self[rownumber][3][1].update_attribs({"class": down_class})
+                self[rownumber][3][1].attribs["class"] = down_class
 
             self[rownumber][3][1][0] = tag.HTMLSymbol("&darr;")
 
@@ -1035,21 +1025,15 @@ class InputTable3(Widget):
                           self.get_formname("down_getfield2"):downget2,
                           self.get_formname("getfield3"):get3
                           }
-            url = self.make_get_url(page, down_url, get_fields, True)
-            self[rownumber][3][1].update_attribs({"href": url})
-
-
-
+            self[rownumber][3][1].attribs["href"] = self.make_get_url(page, down_url, get_fields, True)
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
         """Sets a click event handler"""
-        jscript = """  $("#{ident} a").click(function (e) {{
+        ident=self.get_id()
+        return f"""  $("#{ident} a").click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident = self.get_id())
-        if self._up_jsonurl or self._down_jsonurl:
-            return jscript + self._make_fieldvalues(upurl=self._up_jsonurl, downurl=self._down_jsonurl)
-
+"""
 
 
     @classmethod
