@@ -1,13 +1,9 @@
 
-
 """Contains commonly used paragraphs"""
 
-from string import Template
-import json
 
-from .. import skiboot
 from .. import tag
-from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
+from . import Widget, ClosedWidget, AnchorClickEventMixin, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
 class TagBlock(Widget):
@@ -32,35 +28,33 @@ class TagBlock(Widget):
            dropident - ident or label of target which returns a JSON page, called when a drop occurs here
 
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # container is the widget itself
         self[0] =  ""
-        self._dropurl = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self.tag_name = self.get_field_value('tag')
+        self.tag_name = self.wf.tag
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        # dropurl
-        self._dropurl = skiboot.get_url(self.get_field_value("dropident"), proj_ident=page.proj_ident)
-        # drag
-        drag = self.get_field_value("drag")
-        if drag:
-            self.update_attribs(
-{"draggable":"true",
-"ondragstart":"SKIPOLE.widgets['{ident}'].dragstartfunc(event, '{data}')".format(ident = self.get_id(),
-                                                                                 data = drag)})
-        # drop
-        drop = self.get_field_value("drop")
-        if drop:
-            self.update_attribs(
-{"ondrop":"SKIPOLE.widgets['{ident}'].dropfunc(event, '{data}')".format(ident = self.get_id(), data = drop),
-"ondragover":"SKIPOLE.widgets['{ident}'].allowdropfunc(event)".format(ident = self.get_id())})
+        self.widget_hide(self.wf.hide)
 
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets drop and drag"""
-        if self._dropurl:
-            return self._make_fieldvalues(dropurl=self._dropurl)
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        dropurl = self.get_url(self.wf.dropident)
+        if dropurl:
+            self.jlabels['dropurl'] = dropurl
+
+        # drag
+        drag = self.wf.drag
+        if drag:
+            self.attribs.update(
+{"draggable":"true",
+"ondragstart":f"SKIPOLE.widgets['{self.get_id()}'].dragstartfunc(event, '{drag}')"})
+        # drop
+        drop = self.wf.drop
+        if drop:
+            self.attribs.update(
+{"ondrop":f"SKIPOLE.widgets['{self.get_id()}'].dropfunc(event, '{drop}')",
+"ondragover":f"SKIPOLE.widgets['{self.get_id()}'].allowdropfunc(event)"})
 
 
     @classmethod
@@ -87,17 +81,18 @@ class DivStyleDiv(Widget):
         """A div, containing a div which can have a style set, containing unescaped html
             inner_tag - the tag of the inside element
            style - the style of the inside element"""
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name='div')
         self[0].htmlescaped = False
         self[0].linebreaks=False
         self[0][0] =  ""  # where html is to be set
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self[0].tag_name = self.get_field_value('inner_tag')
-        if self.get_field_value('style'):
-            self[0].update_attribs({"style":self.get_field_value('style')})
-        self[0][0] = self.get_field_value("set_html")
+        self[0].tag_name = self.wf.inner_tag
+        if self.wf.style:
+            self[0].attribs["style"] = self.wf.style
+        self[0][0] = self.wf.set_html
 
     @classmethod
     def description(cls):
@@ -129,36 +124,34 @@ class DivHTML(Widget):
                   if nothing set here, this item will not be set as droppable.
            dropident - ident or label of target which returns a JSON page, called when a drop occurs here"""
 
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = ""  # where the html string is to be set
         self.htmlescaped = False
         self.linebreaks=False
-        self._dropurl = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self[0] = self.get_field_value("set_html")
+        self[0] = self.wf.set_html
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
+        self.widget_hide(self.wf.hide)
         # dropurl
-        self._dropurl = skiboot.get_url(self.get_field_value("dropident"), proj_ident=page.proj_ident)
-        # drag
-        drag = self.get_field_value("drag")
-        if drag:
-            self.update_attribs(
-{"draggable":"true",
-"ondragstart":"SKIPOLE.widgets['{ident}'].dragstartfunc(event, '{data}')".format(ident = self.get_id(),
-                                                                                 data = drag)})
-        # drop
-        drop = self.get_field_value("drop")
-        if drop:
-            self.update_attribs(
-{"ondrop":"SKIPOLE.widgets['{ident}'].dropfunc(event, '{data}')".format(ident = self.get_id(), data = drop),
-"ondragover":"SKIPOLE.widgets['{ident}'].allowdropfunc(event)".format(ident = self.get_id())})
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        dropurl = self.get_url(self.wf.dropident)
+        if dropurl:
+            self.jlabels['dropurl'] = dropurl
 
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets drop and drag"""
-        if self._dropurl:
-            return self._make_fieldvalues(dropurl=self._dropurl)
+        # drag
+        drag = self.wf.drag
+        if drag:
+            self.attribs.update(
+{"draggable":"true",
+"ondragstart":f"SKIPOLE.widgets['{self.get_id()}'].dragstartfunc(event, '{drag}')"})
+        # drop
+        drop = self.wf.drop
+        if drop:
+            self.attribs.update(
+{"ondrop":f"SKIPOLE.widgets['{self.get_id()}'].dropfunc(event, '{drop}')",
+"ondragover":f"SKIPOLE.widgets['{self.get_id()}'].allowdropfunc(event)"})
 
     @classmethod
     def description(cls):
@@ -183,11 +176,12 @@ class PreText(Widget):
         pre_text: The text appearing in the pre tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="pre", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "pre"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self[0] = self.get_field_value("pre_text")
+        self[0] = self.wf.pre_text
 
     @classmethod
     def description(cls):
@@ -211,11 +205,12 @@ class SpanText(Widget):
         span_text: The text appearing in the span tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="span", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "span"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self[0] = self.get_field_value("span_text")
+        self[0] = self.wf.span_text
 
     @classmethod
     def description(cls):
@@ -241,14 +236,15 @@ class TagText(Widget):
         tag_text: The text appearing in the tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self.tag_name = self.get_field_value('tag')
+        self.tag_name = self.wf.tag
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        self[0] = self.get_field_value("tag_text")
+        self.widget_hide(self.wf.hide)
+        self[0] = self.wf.tag_text
 
     @classmethod
     def description(cls):
@@ -274,14 +270,15 @@ class TagUnEscaped(Widget):
         content: The content appearing in the tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = ''
         self.htmlescaped = False
         self.linebreaks=False
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self.tag_name = self.get_field_value('tag')
-        self[0] = self.get_field_value("content")
+        self.tag_name = self.wf.tag
+        self[0] = self.wf.content
 
     @classmethod
     def description(cls):
@@ -305,11 +302,12 @@ class ParaText(Widget):
         para_text: The text appearing in the p tag
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="p", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "p"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        self[0] = self.get_field_value("para_text")
+        self[0] = self.wf.para_text
 
     @classmethod
     def description(cls):
@@ -342,27 +340,27 @@ class DivPara(Widget):
                   new line breaks
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name='p')
         # do not insert <br /> - leave that to pre_line
         self[0].linebreaks=False
         self[0][0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value("pre_line"):
-            self[0].attribs={"style":"white-space: pre-line;"}
-        if self.get_field_value('para_class'):
-            self[0].update_attribs({"class":self.get_field_value('para_class')})
+        if self.wf.pre_line:
+            self[0].attribs["style"] = "white-space: pre-line;"
+        if self.wf.para_class:
+            self[0].attribs["class"] = self.wf.para_class
         # self[0][0] could be set by an error message
         if not self.error_status:
-            self[0][0] = self.get_field_value("para_text")
-        if self.error_status and self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+            self[0][0] = self.wf.para_text
+        if self.error_status and self.wf.error_class:
+            self[0].attribs["class"] = self.wf.error_class
 
-            
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets fieldvalues"""
-        return self._make_fieldvalues('para_class', 'error_class')
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['para_class'] = self.wf.para_class
+        self.jlabels['error_class'] = self.wf.error_class
 
 
     @classmethod
@@ -378,7 +376,7 @@ class DivPara(Widget):
 </div>"""
 
 
-class JSONTextLink(Widget):
+class JSONTextLink(AnchorClickEventMixin, Widget):
     """A div, containing an anchor link (used as a button) followed by a paragraph containing text
        If the button is pressed, and the text is visible - the text is hidden, and button text set to 'button_show_text'.
        If the button is pressed, and the text is hidden - a call to a json file is made which should return the
@@ -406,76 +404,65 @@ class JSONTextLink(Widget):
                   new line breaks
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
-        self[0] = tag.Part(tag_name='a', attribs={'role':'button'})
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
+        self[0] = tag.Part(tag_name='a', attribs={'role':'button', 'href':'#'})
         self[0][0] = ""  # where button text is to be set
         self[1] = tag.Part(tag_name="p")
         # do not insert <br /> - leave that to pre_line
         self[1].linebreaks=False
         self[1][0] =  ""  # where para_text is to be set
-        self._jsonurl = ''
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value('button_class'):
-             self[0].update_attribs({"class":self.get_field_value('button_class')})
-        self[0].update_attribs({"href":"#"})
-        if not self.get_field_value("link_ident"):
+        if self.wf.button_class:
+             self[0].attribs["class"] = self.wf.button_class
+        if not self.wf.link_ident:
             # setting self._error replaces the entire tag
             self._error = "Warning: No link ident"
             return
-        url = skiboot.get_url(self.get_field_value("link_ident"), proj_ident=page.proj_ident)
+        url = self.get_url(self.wf.link_ident)
         if url:
-            get_fields = {self.get_formname("get_field"):self.get_field_value("get_field")}
-            url = self.make_get_url(page, url, get_fields, force_ident=True)
-            self[0].update_attribs({"href": url})
+            get_fields = {self.get_formname("get_field"):self.wf.get_field}
+            self[0].attribs["href"] = self.make_get_url(page, url, get_fields, force_ident=True)
         else:
             self._error = "Warning: Invalid link ident"
             return
+ 
+        if self.wf.para_class:
+            self[1].attribs["class"] = self.wf.para_class
 
-        para_style = {}
-
-        if self.get_field_value("para_class"):
-            para_style["class"] = self.get_field_value("para_class")
-
-        if self.get_field_value("hide") and (not self.error_status):
-            self[0][0] = self.get_field_value("button_show_text")
-            if self.get_field_value("pre_line"):
-                para_style["style"] = "display:none;white-space: pre-line;"
+        if self.wf.hide and (not self.error_status):
+            self[0][0] = self.wf.button_show_text
+            if self.wf.pre_line:
+                self[1].attribs["style"] = "display:none;white-space: pre-line;"
             else:
-                para_style["style"] = "display:none;"
+                self[1].attribs["style"] = "display:none;"
         else:
-            self[0][0] = self.get_field_value("button_hide_text")
-            if self.get_field_value("pre_line"):
-                para_style["style"] = "white-space: pre-line;"
-
-        if para_style:
-            self[1].attribs=para_style
+            self[0][0] = self.wf.button_hide_text
+            if self.wf.pre_line:
+                self[1].attribs["style"] = "white-space: pre-line;"
 
         if not self.error_status:
             # only set para_text if an error is not already there
-            self[1][0] = self.get_field_value("para_text")
-        if not self.get_field_value("json_ident"):
+            self[1][0] = self.wf.para_text
+        if not self.self.wf.json_ident:
             # setting self._error replaces the entire tag
             self._error = "Warning: No link ident to JSON"
             return
-        self._jsonurl = skiboot.get_url(self.get_field_value("json_ident"),  proj_ident=page.proj_ident)
-        if not self._jsonurl:
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['button_show_text'] = self.wf.button_show_text
+        self.jlabels['button_hide_text'] = self.wf.button_hide_text
+
+        jsonurl = self.get_url(self.wf.json_ident)
+        if not jsonurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
+        self.jlabels['url'] = jsonurl
 
 
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets a click event handler"""
-        if not self._jsonurl:
-            return
-        jscript = """  $("#{ident} a").click(function (e) {{
-    SKIPOLE.widgets['{ident}'].eventfunc(e);
-    }});
-""".format(ident = self.get_id())
-        return jscript + self._make_fieldvalues( 'button_show_text', 'button_hide_text', url=self._jsonurl)
-
     @classmethod
     def description(cls):
         """Returns a text string to illustrate the widget"""
@@ -506,6 +493,7 @@ class JSONDivLink(Widget):
                         'button_show_text':FieldArg("text", "Show"),
                         'button_hide_text':FieldArg("text", "Hide"),
                         'button_class':FieldArg("cssclass", ""),
+                        'button_style':FieldArg("cssstyle", ""),
                         'json_ident':FieldArg("url", ''),
                         'link_ident':FieldArg("url", 'no_javascript'),
                         'get_field':FieldArg("text", "", valdt=True),
@@ -518,68 +506,69 @@ class JSONDivLink(Widget):
         div_content: The content appearing in the div
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
-        self[0] = tag.Part(tag_name='a', attribs={'role':'button'})
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
+        self[0] = tag.Part(tag_name='a', attribs={'role':'button', 'href':'#'})
         self[0][0] = ""  # where button text is to be set
         self[1] = tag.Part(tag_name='div')
         self[1][0] =  ""  # where div_content is to be set
-        self._jsonurl = ''
-
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value('button_class'):
-             self[0].update_attribs({"class":self.get_field_value('button_class')})
-        self[0].update_attribs({"href":"#"})
-        if self.get_field_value("link_ident"):
-            url = skiboot.get_url(self.get_field_value("link_ident"), proj_ident=page.proj_ident)
+        if self.wf.button_class:
+             self[0].attribs["class"] = self.wf.button_class
+        if self.wf.button_style:
+             self[0].attribs["style"] = self.wf.button_style
+        if self.wf.link_ident:
+            url = self.get_url(self.wf.link_ident)
             if url:
-                get_fields = {self.get_formname("get_field"):self.get_field_value("get_field")}
-                url = self.make_get_url(page, url, get_fields, force_ident=True)
-                self[0].update_attribs({"href": url})
+                get_fields = {self.get_formname("get_field"):self.wf.get_field}
+                self[0].attribs["href"] = self.make_get_url(page, url, get_fields, force_ident=True)
 
-        if self.get_field_value("hide") and (not self.error_status):
-            self[1].attribs={"style":"display:none;"}
-            self[0][0] = self.get_field_value("button_show_text")
+        if self.wf.hide and (not self.error_status):
+            self[1].attribs["style"] = "display:none;"
+            self[0][0] = self.wf.button_show_text
         else:
-            self[1].attribs={}
-            self[0][0] = self.get_field_value("button_hide_text")
-        if self.get_field_value('div_class'):
-            self[1].update_attribs({"class":self.get_field_value('div_class')})
+            self[0][0] = self.wf.button_hide_text
+        if self.wf.div_class:
+            self[1].attribs["class"] = self.wf.div_class
         if not self.error_status:
             # only set div_content if an error is not already there
-            self[1][0] = self.get_field_value("div_content")
-        if not self.get_field_value("htmlescaped"):
+            self[1][0] = self.wf.div_content
+        if not self.wf.htmlescaped:
             # set the div to not escape its contents
             self[1].htmlescaped = False
         # set an id in the button and div
-        self[0].insert_id()
-        self[1].insert_id()
-        if not self.get_field_value("json_ident"):
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['buttonident'] = self[0].insert_id()
+        self.jlabels['divident'] = self[1].insert_id()
+
+        self.jlabels['button_show_text'] = self.wf.button_show_text
+        self.jlabels['button_hide_text'] = self.wf.button_hide_text
+        self.jlabels['get_field'] = self.wf.get_field
+
+        self.jlabels['htmlescaped'] = "text" if self.wf.htmlescaped else "html"
+
+        if not self.wf.json_ident:
             # setting self._error replaces the entire tag
             self._error = "Warning: No link ident to JSON"
             return
-        self._jsonurl = skiboot.get_url(self.get_field_value("json_ident"),  proj_ident=page.proj_ident)
-        if not self._jsonurl:
+        jsonurl = self.get_url(self.wf.json_ident)
+        if not jsonurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
+        self.jlabels['url'] = jsonurl
 
 
     def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets a click event handler"""
-        jscript = """  $("#{buttonident}").click(function (e) {{
+        """Sets a click event handler on the button"""
+        ident = self.get_id()
+        buttonident = self[0].get_id() # ident of the button
+        return f"""  $("#{buttonident}").click(function (e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(
-        ident = self.get_id(),
-        buttonident = self[0].get_id(), # ident of the button
-        )
-        return jscript + self._make_fieldvalues("button_hide_text", "button_show_text", "get_field",
-                                                htmlescaped = "text" if self.get_field_value("htmlescaped") else "html",
-                                                divident = self[1].get_id(),
-                                                buttonident = self[0].get_id(),
-                                                url = self._jsonurl
-                                                )
+"""
+
 
     @classmethod
     def description(cls):
@@ -589,7 +578,7 @@ class JSONDivLink(Widget):
   <a href="#" role="button">  <!-- with class button_class -->
     <!-- set with button_show_text or button_hide_text -->
   </a>
-  <div>  <!-- with class div_class, style div_style, and either hidden or not, depending on hide -->
+  <div>  <!-- with class div_class, and either hidden or not, depending on hide -->
     <!-- div_content shown as either text or html -->
   </div>
 </div>"""
@@ -622,33 +611,36 @@ class TextBlockPara(Widget):
                      replaces widget_class on error.
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="p", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "p"
         self[0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
         if not self.error_status:
-            linebreaks = bool(self.get_field_value('linebreaks'))
-            if self.get_field_value('text_replaceblock'):
+            linebreaks = bool(self.wf.linebreaks)
+            if self.wf.text_replaceblock:
                 # no textblock, just the replacement text
-                self[0] = self.get_field_value('text_replaceblock')
+                self[0] = self.wf.text_replaceblock
                 if not linebreaks:
                     self.linebreaks = False             
             else:
                 # define the textblock
-                tblock = self.get_field_value("textblock_ref")
-                tblock.project = self.get_field_value('textblock_project')
-                tblock.failmessage = self.get_field_value('text_refnotfound')
+                tblock = self.wf.textblock_ref
+                tblock.project = self.wf.textblock_project
+                tblock.failmessage = self.wf.text_refnotfound
                 tblock.linebreaks = linebreaks
-                if self.get_field_value('replace_strings'):
-                    tblock.replace_strings = self.get_field_value('replace_strings')
+                if self.wf.replace_strings:
+                    tblock.replace_strings = self.wf.replace_strings
                 # place it at location 0
                 self[0] = tblock
-        if self.error_status and self.get_field_value('error_class'):
-            self.update_attribs({"class":self.get_field_value('error_class')})
+        if self.error_status and self.wf.error_class:
+            self.attribs["class"] = self.wf.error_class
+
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['widget_class'] = self.wf.widget_class
+        self.jlabels['error_class'] = self.wf.error_class
+        self.jlabels['linebreaks'] = self.wf.linebreaks
 
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sets fieldvalues"""
-        return self._make_fieldvalues('widget_class','error_class','linebreaks')
 
     @classmethod
     def description(cls):
@@ -682,21 +674,22 @@ class TextBlockDiv(Widget):
         content_replaceblock: content set here will replace the textblock
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = ''
         self.htmlescaped = False
         self.linebreaks = False
 
     def _build(self, page, ident_list, environ, call_data, lang):
         # define the textblock
-        tblock = self.get_field_value("textblock_ref")
-        tblock.project = self.get_field_value('textblock_project')
-        tblock.failmessage = self.get_field_value('content_refnotfound')
+        tblock = self.wf.textblock_ref
+        tblock.project = self.wf.textblock_project
+        tblock.failmessage = self.wf.content_refnotfound
         tblock.escape = False
         tblock.linebreaks = False
         # place it at location 0
-        if self.get_field_value("content_replaceblock"):
-            self[0] = self.get_field_value("content_replaceblock")
+        if self.wf.content_replaceblock:
+            self[0] = self.wf.content_replaceblock
         else:
             self[0] = tblock
 
@@ -732,7 +725,8 @@ class ShowPara1(Widget):
         pre_line: If True, sets style="white-space: pre-line;" into the paragraph which preserves
                   new line breaks
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # The para_text
         self[0] = tag.Part(tag_name="p")
         # do not insert <br /> - leave that to pre_line
@@ -742,28 +736,24 @@ class ShowPara1(Widget):
         self[1][0][0] = ''
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        para_style = {}
 
-        if self.get_field_value("para_class"):
-            para_style["class"] = self.get_field_value("para_class")
+        if self.wf.para_class:
+            self[0].attribs["class"] = self.wf.para_class
 
-        if self.get_field_value("show_para"):
-            if self.get_field_value("pre_line"):
-                para_style["style"] = "white-space: pre-line;"
+        if self.wf.show_para:
+            if self.wf.pre_line:
+                self[0].attribs["style"] = "white-space: pre-line;"
         else:
-            if self.get_field_value("pre_line"):
-                para_style["style"] = "display:none;white-space: pre-line;"
+            if self.wf.pre_line:
+                self[0].attribs["style"] = "display:none;white-space: pre-line;"
             else:
-                para_style["style"] = "display:none;"
+                self[0].attribs["style"] = "display:none;"
 
-        if para_style:
-            self[0].attribs=para_style
-
-        self[0][0] = self.get_field_value("para_text")
+        self[0][0] = self.wf.para_text
         if self.error_status:
-            self[1].del_one_attrib("style")
-        if self.get_field_value('error_class'):
-            self[1].update_attribs({"class":self.get_field_value('error_class')})
+            del self[1].attribs["style"]
+        if self.wf.error_class:
+            self[1].attribs["class"] = self.wf.error_class
 
     @classmethod
     def description(cls):
@@ -801,7 +791,8 @@ class ShowPara2(Widget):
                   new line breaks
         """
         # pass fields to Widget
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         self[0] = tag.Part(tag_name='div', attribs={"onclick":"this.parentElement.style.display='none'"})
         self[0][0] = tag.HTMLSymbol("&times;")
         self[1] = tag.Part(tag_name='div')
@@ -812,17 +803,17 @@ class ShowPara2(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
+        self.widget_hide(self.wf.hide)
         # define the text paragraph
-        self[1][0][0] = self.get_field_value("para_text")
-        if self.get_field_value("inner_class"):
-            self[1].update_attribs({"class":self.get_field_value("inner_class")})
-        if self.get_field_value("close_class"):
-            self[0].update_attribs({"class":self.get_field_value("close_class")})
-        if self.get_field_value("close_style"):
-            self[0].update_attribs({"style":self.get_field_value("close_style")})
-        if self.get_field_value("pre_line"):
-            self[1][0].attribs = {"style":"white-space: pre-line;"}
+        self[1][0][0] = self.wf.para_text
+        if self.wf.inner_class:
+            self[1].attribs["class"] = self.wf.inner_class
+        if self.wf.close_class:
+            self[0].attribs["class"] = self.wf.close_class
+        if self.wf.close_style:
+            self[0].attribs["style"] = self.wf.close_style
+        if self.wf.pre_line:
+            self[1][0].attribs["style"] = "white-space: pre-line;"
 
     @classmethod
     def description(cls):
