@@ -16,8 +16,6 @@ class Arrow1(ClosedWidget):
     # This class does not display any error messages
     display_errors = False
 
-    _points = ((49,1), (50,1), (98,30), (98,32), (60,32), (60,98), (39,98), (39,32), (1,32), (1,30))
-
     arg_descriptions = {'fill':FieldArg("text", "none", jsonset=True),
                         'stroke':FieldArg("text", "black", jsonset=True),
                         'transform':FieldArg("text", "", jsonset=True),
@@ -31,24 +29,20 @@ class Arrow1(ClosedWidget):
         stroke_width: The outline edge thickness
         transform: The svg transform object, use it to scals and rotate
         """
-        ClosedWidget.__init__(self, name=name, tag_name="polygon", brief=brief, **field_args)
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "polygon"
+        self.attribs["points"] = "49,1 50,1 98,30 98,32 60,32 60,98 39,98 39,32 1,32 1,30"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "create arrow"                  
-        attrib_points = ""
-        for p in self._points:
-            point = "%s, %s " % p
-            attrib_points += point
-        self.update_attribs({"points":attrib_points})
-
-        if self.get_field_value("fill"):
-            self.update_attribs({"fill":self.get_field_value("fill")})
-        if self.get_field_value("stroke"):
-            self.update_attribs({"stroke":self.get_field_value("stroke")})
-        if self.get_field_value("transform"):
-            self.update_attribs({"transform":self.get_field_value("transform")})
-        if self.get_field_value("stroke_width"):
-            self.update_attribs({"stroke-width":self.get_field_value("stroke_width")})
+        if self.wf.fill:
+            self.attribs["fill"] = self.wf.fill
+        if self.wf.stroke:
+            self.attribs["stroke"] = self.wf.stroke
+        if self.wf.transform:
+            self.attribs["transform"] = self.wf.transform
+        if self.wf.stroke_width:
+            self.attribs["stroke-width"] = self.wf.stroke_width
 
     @classmethod
     def description(cls):
@@ -65,8 +59,6 @@ class Arrow2(ClosedWidget):
     # This class does not display any error messages
     display_errors = False
 
-    _points = ((24,1), (25,1), (49,50), (49,52), (30,52), (30,198), (19,198), (19,52), (1,52), (1,50))
-
     arg_descriptions = {'fill':FieldArg("text", "none", jsonset=True),
                         'stroke':FieldArg("text", "black", jsonset=True),
                         'transform':FieldArg("text", "", jsonset=True),
@@ -80,24 +72,20 @@ class Arrow2(ClosedWidget):
         stroke_width: The outline edge thickness
         transform: The svg transform object, use it to scals and rotate
         """
-        ClosedWidget.__init__(self, name=name, tag_name="polygon", brief=brief, **field_args)
+        ClosedWidget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "polygon"
+        self.attribs["points"] = "24,1 25,1 49,50 49,52 30,52 30,198 19,198 19,52 1,52 1,50"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "create arrow"                  
-        attrib_points = ""
-        for p in self._points:
-            point = "%s, %s " % p
-            attrib_points += point
-        self.update_attribs({"points":attrib_points})
-
-        if self.get_field_value("fill"):
-            self.update_attribs({"fill":self.get_field_value("fill")})
-        if self.get_field_value("stroke"):
-            self.update_attribs({"stroke":self.get_field_value("stroke")})
-        if self.get_field_value("transform"):
-            self.update_attribs({"transform":self.get_field_value("transform")})
-        if self.get_field_value("stroke_width"):
-            self.update_attribs({"stroke-width":self.get_field_value("stroke_width")})
+        if self.wf.fill:
+            self.attribs["fill"] = self.wf.fill
+        if self.wf.stroke:
+            self.attribs["stroke"] = self.wf.stroke
+        if self.wf.transform:
+            self.attribs["transform"] = self.wf.transform
+        if self.wf.stroke_width:
+            self.attribs["stroke-width"] = self.wf.stroke_width
 
     @classmethod
     def description(cls):
@@ -111,8 +99,6 @@ class Vertical1(Widget):
     # This class does not display any error messages
     display_errors = False
 
-    _points = ((110,49), (110,50), (81,98), (79,98), (79,60), (13,60), (13,39), (79,39), (79,1), (81,1))
-
     arg_descriptions = {
                         'transform':FieldArg("text", "", jsonset=True),
                         'arrow_fill':FieldArg("text", "blue", jsonset=True),
@@ -152,7 +138,8 @@ class Vertical1(Widget):
 
     def __init__(self, name=None, brief='', **field_args):
         """A g element which holds a vertical scale and arrow, held in a 700 high x 250 wide space"""
-        Widget.__init__(self, name=name, tag_name="g", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "g"
         self[0] = tag.ClosedPart(tag_name='rect', attribs={"x":"100",
                                                            "y":"1",
                                                            "rx":"2",
@@ -162,10 +149,8 @@ class Vertical1(Widget):
                                                            "fill":"white",
                                                            "stroke":"black",
                                                            "stroke-width":"1"})
-        arrow_points = ""
-        for p in self._points:
-            point = "%s, %s " % p
-            arrow_points += point
+        arrow_points = "110,49 110,50 81,98 79,98 79,60 13,60 13,39 79,39 79,1 81,1"
+
         self[1] = tag.ClosedPart(tag_name='polygon', attribs={
                                                            "fill":"white",
                                                            "stroke":"black",
@@ -181,20 +166,20 @@ class Vertical1(Widget):
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value("transform"):
-            self.update_attribs({"transform":self.get_field_value("transform")})
+        if self.wf.transform:
+            self.attribs["transform"] = self.wf.transform
 
-        font_family = self.get_field_value("font_family")
+        font_family = self.wf.font_family
         if not font_family:
             font_family = "arial"
 
-        if self.get_field_value("arrow_fill"):
-            self[1].update_attribs({"fill":self.get_field_value("arrow_fill")})
+        if self.wf.arrow_fill:
+            self[1].attribs["fill"] = self.wf.arrow_fill
         # make the scale
-        minscale, maxscale = self._make_scale(self.get_field_value("minimum"),
-                                              self.get_field_value("maximum"),
-                                              self.get_field_value("smallintervals"),
-                                              self.get_field_value("largeintervals"))
+        minscale, maxscale = self._make_scale(self.wf.minimum,
+                                              self.wf.maximum,
+                                              self.wf.smallintervals,
+                                              self.wf.largeintervals)
 
         # small lines
         minitems = len(minscale)
@@ -236,20 +221,20 @@ class Vertical1(Widget):
             n += 1
 
         # now place arrow at the measurement point
-        measurement = Decimal(self.get_field_value("measurement"))
+        measurement = Decimal(self.wf.measurement)
         self._minvalue = maxscale[0]
         self._maxvalue = maxscale[-1]
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['maxvalue'] = self._maxvalue
+        self.jlabels['minvalue'] = self._minvalue
+
         if measurement >= self._maxvalue:
             return
         if measurement <= self._minvalue:
-            self[1].update_attribs({"transform" : "translate(0, 600)"})
+            self[1].attribs["transform"] = "translate(0, 600)"
             return
         m = Decimal('600.0') - (measurement - self._minvalue)*600/(self._maxvalue-self._minvalue)
-        self[1].update_attribs({"transform" : "translate(0, %s)" % (m,)})
-
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sends scaling factor for mapping measurement to scale"""
-        return self._make_fieldvalues(maxvalue=str(self._maxvalue), minvalue=str(self._minvalue))
+        self[1].attribs["transform"] = f"translate(0, {m})"
 
 
     @classmethod
@@ -270,8 +255,6 @@ class Traditional1(Widget):
     # This class does not display any error messages
     display_errors = False
 
-    _points = ((24,1), (25,1), (49,50), (49,52), (30,52), (30,198), (19,198), (19,52), (1,52), (1,50))
-
     arg_descriptions = {
                         'transform':FieldArg("text", "translate(10,10)", jsonset=True),
                         'minimum':FieldArg("text", "0"),
@@ -311,12 +294,12 @@ class Traditional1(Widget):
 
     def __init__(self, name=None, brief='', **field_args):
         """A g element which holds the scale and arrow, held in a 400 high x 700 wide space"""
-        Widget.__init__(self, name=name, tag_name="g", brief=brief, **field_args)
-
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "g"
         # A path which holds the curved shape which will contain the meter
 
         # the angle of the white backing is 140 degrees, this makes an
-        # angle of 20 degrees to the horizonta. So get this in radians
+        # angle of 20 degrees to the horizontal. So get this in radians
         back_horizontal_angle = math.radians(20.0)
 
         # The scale
@@ -388,17 +371,18 @@ A %s %s 0 0 1 %s %s
         self[1] = tag.ClosedPart(tag_name='path',
                                  attribs={ "fill":"none", "stroke":"black", "stroke-width":"2", "d":scale_data})
 
-
         # The arrow points
-        arrow_points = ""
+        _points = ((24,1), (25,1), (49,50), (49,52), (30,52), (30,198), (19,198), (19,52), (1,52), (1,50))
         # move all points to the right and down,
         # note 24.5 is x distance to arrow point
         x_move = cx - 24.5
         # moves arrow down to just touch the scale
         y_move = cy - r2
-        for p in self._points:
-            point = "%s, %s " % (p[0] + x_move, p[1] + y_move)
-            arrow_points += point
+        # get a generator producing strings of the form "x,y"
+        movedpoints = (str(p[0] + x_move) + "," + str(p[1] + y_move) for p in _points)
+        # and get a string of the points separated by spaces
+        arrow_points = " ".join(movedpoints)
+
         self[2] = tag.ClosedPart(tag_name='polygon', attribs={
                                                            "fill":"black",
                                                            "stroke":"grey",
@@ -417,21 +401,21 @@ A %s %s 0 0 1 %s %s
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value("transform"):
-            self.update_attribs({"transform":self.get_field_value("transform")})
+        if self.wf.transform:
+            self.attribs["transform"] = self.wf.transform
 
-        font_family = self.get_field_value("font_family")
+        font_family = self.wf.font_family
         if not font_family:
             font_family = "arial"
 
-        if self.get_field_value("arrow_stroke"):
-            self[2].update_attribs({"stroke":self.get_field_value("arrow_stroke")})
-            self[3].update_attribs({"stroke":self.get_field_value("arrow_stroke")})
+        if self.wf.arrow_stroke:
+            self[2].attribs["stroke"] = self.wf.arrow_stroke
+            self[3].attribs["stroke"] = self.wf.arrow_stroke
         # make the scale
-        minscale, maxscale = self._make_scale(self.get_field_value("minimum"),
-                                              self.get_field_value("maximum"),
-                                              self.get_field_value("smallintervals"),
-                                              self.get_field_value("largeintervals"))
+        minscale, maxscale = self._make_scale(self.wf.minimum,
+                                              self.wf.maximum,
+                                              self.wf.smallintervals,
+                                              self.wf.largeintervals)
 
         # start angle is 180 - 120 / 2 normally 30
         start_angle = (Decimal('180') - self._scale_angle)/Decimal('2')
@@ -493,26 +477,24 @@ A %s %s 0 0 1 %s %s
             n += 1
 
         # now place arrow at the measurement point
-        measurement = Decimal(self.get_field_value("measurement"))
+        measurement = Decimal(self.wf.measurement)
         self._minvalue = maxscale[0]
         self._maxvalue = maxscale[-1]
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['maxvalue'] = self._maxvalue
+        self.jlabels['minvalue'] = self._minvalue
+
         centre_string = " " + str(self._cx) + " " + str(self._cy) + ")"
         if measurement >= self._maxvalue:
-            self[2].update_attribs({"transform" : "rotate(" + str(self._scale_angle/2) + centre_string})
+            self[2].attribs["transform"] = "rotate(" + str(self._scale_angle/2) + centre_string
             return
         if measurement <= self._minvalue:
-            self[2].update_attribs({"transform" : "rotate(-" + str(self._scale_angle/2) + centre_string})
+            self[2].attribs["transform"] = "rotate(-" + str(self._scale_angle/2) + centre_string
             return
 
         measurement_angle = (measurement - self._minvalue)*self._scale_angle/(self._maxvalue-self._minvalue) - self._scale_angle/2
 
-        rotate_string = "rotate(" + str(measurement_angle) + centre_string
-        self[2].update_attribs({"transform" : rotate_string})
-
-
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sends scaling factor for mapping measurement to scale"""
-        return self._make_fieldvalues(maxvalue=str(self._maxvalue), minvalue=str(self._minvalue))
+        self[2].attribs["transform"] = "rotate(" + str(measurement_angle) + centre_string
 
 
     @classmethod
@@ -545,7 +527,8 @@ class Angle(Widget):
 
     def __init__(self, name=None, brief='', **field_args):
         """A g element which holds the compass, held in a 500 high x 500 wide space"""
-        Widget.__init__(self, name=name, tag_name="g", brief=brief, **field_args) 
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "g"
         # insert a circle at arrow hub, of radius 240
         self[0] = tag.ClosedPart(tag_name='circle', attribs={
                                                        "cx": "250",
@@ -580,13 +563,13 @@ class Angle(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "create compass"                  
-        if self.get_field_value("transform"):
-            self.update_attribs({"transform":self.get_field_value("transform")})
-        font_family = self.get_field_value("font_family")
+        if self.wf.transform:
+            self.attribs["transform"] = self.wf.transform
+        font_family = self.wf.font_family
         if not font_family:
             font_family = "arial"
-        self._scale_units = self.get_field_value("scale_units")
-        measurement = Decimal(self.get_field_value("measurement"))
+        self._scale_units = self.wf.scale_units
+        measurement = Decimal(self.wf.measurement)
         rotate_string = ''
         if self._scale_units == '24':
             self._draw_24(font_family)
@@ -605,7 +588,9 @@ class Angle(Widget):
             if (measurement > 0) and (measurement < 360):
                 rotate_string = "rotate(" + str(measurement) + " 250 250)"
         if rotate_string:
-            self[3].update_attribs({"transform" : rotate_string})
+            self[3].attribs["transform"] = rotate_string
+        # any label:value added to self.jlabels will be set in a javascript fieldvalues attribute for the widget
+        self.jlabels['scale_units'] = self._scale_units
 
 
     def _draw_24(self, font_family):
@@ -828,10 +813,6 @@ class Angle(Widget):
                                                                 'stroke-width':"1",
                                                                 'transform' : minor_rotate  })  )
 
-    def _build_js(self, page, ident_list, environ, call_data, lang):
-        """Sends scaling factor for mapping measurement to scale"""
-        return self._make_fieldvalues(scale_units=self._scale_units)
-
 
     @classmethod
     def description(cls):
