@@ -1,6 +1,6 @@
 
 
-from .. import skiboot, tag
+from .. import tag
 from . import Widget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -28,12 +28,13 @@ class ColorTable1(Widget):
         titles: List of text used as headers, gives the number of columns
         contents: list of lists, 0:text in the table, 1:the text color, 2:the background color
         """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        titles = self.get_field_value("titles")
-        contents = self.get_field_value("contents")
+        titles = self.wf.titles
+        contents = self.wf.contents
         cols = len(titles)
         if (not contents) or (not cols):
             return
@@ -42,18 +43,18 @@ class ColorTable1(Widget):
             return
 
         # set even row class
-        if self.get_field_value('even_class'):
-            even = self.get_field_value('even_class')
+        if self.wf.even_class:
+            even = self.wf.even_class
         else:
             even = ''
         # set odd row class
-        if self.get_field_value('odd_class'):
-            odd = self.get_field_value('odd_class')
+        if self.wf.odd_class:
+            odd = self.wf.odd_class
         else:
             odd = ''
 
-        if self.get_field_value('header_class'):
-            self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+        if self.wf.header_class:
+            self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
         else:
             self[0] = tag.Part(tag_name='tr')
 
@@ -130,29 +131,30 @@ class TwoColTable1(Widget):
         col1 : A list of text strings to place in the first column
         col2: A list of text strings to place in the second column
         """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        col_list1 = self.get_field_value("col1")
-        col_list2 = self.get_field_value("col2")
+        col_list1 = self.wf.col1
+        col_list2 = self.wf.col2
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2'):
+        if self.wf.title1 or self.wf.title2:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
         # set even row colour
-        if self.get_field_value('even_class'):
-            even = self.get_field_value('even_class')
+        if self.wf.even_class:
+            even = self.wf.even_class
         else:
             even = ''
         # set odd row colour
-        if self.get_field_value('odd_class'):
-            odd = self.get_field_value('odd_class')
+        if self.wf.odd_class:
+            odd = self.wf.odd_class
         else:
             odd = ''
         # create rows
@@ -224,31 +226,32 @@ class ThreeColTable1(Widget):
         col2: A list of text strings to place in the second column
         col3: A list of text strings to place in the third column
         """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        col_list1 = self.get_field_value("col1")
-        col_list2 = self.get_field_value("col2")
-        col_list3 = self.get_field_value("col3")
+        col_list1 = self.wf.col1
+        col_list2 = self.wf.col2
+        col_list3 = self.wf.col3
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2') or self.get_field_value('title3'):
+        if self.wf.title1 or self.wf.title2 or self.wf.title3:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
-            self[0][2] = tag.Part(tag_name='th', text = self.get_field_value('title3'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
+            self[0][2] = tag.Part(tag_name='th', text = self.wf.title3)
         # set even row colour
-        if self.get_field_value('even_class'):
-            even = self.get_field_value('even_class')
+        if self.wf.even_class:
+            even = self.wf.even_class
         else:
             even = ''
         # set odd row colour
-        if self.get_field_value('odd_class'):
-            odd = self.get_field_value('odd_class')
+        if self.wf.odd_class:
+            odd = self.wf.odd_class
         else:
             odd = ''
         # create rows
@@ -318,29 +321,30 @@ class TextBlockTable2(Widget):
         contents: col 0 is the text to place in the first column,
                   col 1 is the textblock to place in the second colum
         """
-        Widget.__init__(self, name=name, tag_name="table", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "table"
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "Build the table"
-        fieldtable = self.get_field_value("contents")
+        fieldtable = self.wf.contents
         header = 0
-        if self.get_field_value('title1') or self.get_field_value('title2'):
+        if self.wf.title1 or self.wf.title2:
             header = 1
-            if self.get_field_value('header_class'):
-                self[0] = tag.Part(tag_name='tr', attribs={"class":self.get_field_value('header_class')})
+            if self.wf.header_class:
+                self[0] = tag.Part(tag_name='tr', attribs={"class":self.wf.header_class})
             else:
                 self[0] = tag.Part(tag_name='tr')
-            self[0][0] = tag.Part(tag_name='th', text = self.get_field_value('title1'))
-            self[0][1] = tag.Part(tag_name='th', text = self.get_field_value('title2'))
+            self[0][0] = tag.Part(tag_name='th', text = self.wf.title1)
+            self[0][1] = tag.Part(tag_name='th', text = self.wf.title2)
         # set even row colour
-        if self.get_field_value('even_class'):
-            even = self.get_field_value('even_class')
+        if self.wf.even_class:
+            even = self.wf.even_class
         else:
             even = ''
         # set odd row colour
-        if self.get_field_value('odd_class'):
-            odd = self.get_field_value('odd_class')
+        if self.wf.odd_class:
+            odd = self.wf.odd_class
         else:
             odd = ''
         # create rows
@@ -391,6 +395,7 @@ class ButtonTextBlockTable1(Widget):
                         'hidden_field4':FieldArg("text", valdt=True, jsonset=True),
                         'buttons':FieldArgTable(['text', 'textblock_ref'], valdt=True),
                         'button_class':FieldArg("cssclass", ''),
+                        'button_style':FieldArg("cssstyle", ''),
                         'error_class':FieldArg("cssclass", ''),
                         'table_class':FieldArg("cssclass", '')
                         }
@@ -409,7 +414,8 @@ class ButtonTextBlockTable1(Widget):
                  text is the values on the button, and which will be returned
                  textbock_ref is the reference of the textblock appearing in the second column
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # hidden error
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -422,52 +428,47 @@ class ButtonTextBlockTable1(Widget):
 
 
     def _build(self, page, ident_list, environ, call_data, lang):
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        if self.wf.error_class:
+            self[0].attribs["class"] = self.wf.error_class
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
+        self[1].attribs["action"] = actionurl
 
         # build the table
         table = self[1][0]
-        if self.get_field_value('table_class'):
-            table.update_attribs({"class":self.get_field_value('table_class')})
+        if self.wf.table_class:
+            table.attribs["class"] = self.wf.table_class
 
-        title1 = self.get_field_value("col1_button_title")
-        title2 = self.get_field_value("col2_text_title")
+        title1 = self.wf.col1_button_title
+        title2 = self.wf.col2_text_title
 
         # set even row colour
-        if self.get_field_value('even_class'):
-            evenclass = self.get_field_value('even_class')
+        if self.wf.even_class:
+            evenclass = self.wf.even_class
         else:
             evenclass = ''
         # set odd row colour
-        if self.get_field_value('odd_class'):
-            oddclass = self.get_field_value('odd_class')
+        if self.wf.odd_class:
+            oddclass = self.wf.odd_class
         else:
             oddclass = ''
         # set header row colour
-        if self.get_field_value('header_class'):
-            headerclass = self.get_field_value('header_class')
+        if self.wf.header_class:
+            headerclass = self.wf.header_class
         else:
             headerclass = ''
-        # set button class
-        if self.get_field_value('button_class'):
-            buttonclass = self.get_field_value('button_class')
-        else:
-            buttonclass = ''
 
-        rows = self.get_field_value('buttons')
+        rows = self.wf.buttons
 
         for num, row in enumerate(rows):
             if title1 or title2:
@@ -497,17 +498,15 @@ class ButtonTextBlockTable1(Widget):
             table[rownumber][0] = tag.Part(tag_name="td")
             table[rownumber][1] = tag.Part(tag_name="td")
             # submit button
-            if buttonclass:
-                table[rownumber][0][0]= tag.ClosedPart(tag_name="input",
-                                                    attribs = {"value":row[0],
-                                                    "type":"submit",
-                                                    "name":self.get_formname('buttons'),
-                                                    "class":buttonclass})
-            else:
-                table[rownumber][0][0]= tag.ClosedPart(tag_name="input",
-                                                    attribs = {"value":row[0],
-                                                    "type":"submit",
-                                                    "name":self.get_formname('buttons')})
+            table[rownumber][0][0]= tag.ClosedPart(tag_name="input",
+                                                attribs = {"value":row[0],
+                                                "type":"submit",
+                                                "name":self.get_formname('buttons')})
+            if self.wf.button_class:
+                table[rownumber][0][0].attribs["class"] = self.wf.button_class
+            if self.wf.button_style:
+                table[rownumber][0][0].attribs["style"] = self.wf.button_style
+
             # textblock
             table[rownumber][1][0] = row[1]
 
