@@ -4,7 +4,7 @@
 """Contains widgets for uploading files"""
 
 
-from .. import skiboot, tag, excepts
+from .. import tag
 from . import Widget, ClosedWidget, FieldArg, FieldArgList, FieldArgTable, FieldArgDict
 
 
@@ -54,7 +54,8 @@ class SubmitUploadFile1(Widget):
         hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
         hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # hidden error
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -78,68 +79,60 @@ class SubmitUploadFile1(Widget):
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self.widget_hide(self.wf.hide)
+        if self.wf.error_class:
+            self[0].attribs["class"] = self.wf.error_class
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
+        self[1].attribs["action"] = actionurl
 
         # file button
         # the div holding file label and button
-        if self.get_field_value('fileinputdiv_class'):
-            self[1][0].attribs = {"class": self.get_field_value('fileinputdiv_class')}
-        if self.get_field_value('filebutton_label'):
-            self[1][0][0][0] = self.get_field_value('filebutton_label')
-        if self.get_field_value('filebuttonlabel_class'):
-            self[1][0][0].attribs = {"class": self.get_field_value('filebuttonlabel_class')}
+        if self.wf.fileinputdiv_class:
+            self[1][0].attribs["class"] = self.wf.fileinputdiv_class
+        if self.wf.filebutton_label:
+            self[1][0][0][0] = self.wf.filebutton_label
+        if self.wf.filebuttonlabel_class:
+            self[1][0][0].attribs["class"] = self.wf.filebuttonlabel_class
 
-        # set an id in the file button for the 'label for' tag
-        self[1][0][1].insert_id()
+        # set an id in the file button for the 'label for' tag      
+        # and set the label 'for' attribute
+        self[1][0][0].attribs['for'] = self[1][0][1].insert_id()
 
-        self[1][0][1].update_attribs({"name":self.get_formname('action')})
+        self[1][0][1].attribs["name"] = self.get_formname('action')
 
-        if self.get_field_value('required'):
-            self[1][0][1].update_attribs({"required":"required"})
-        if self.get_field_value('filebutton_class'):
-            self[1][0][1].update_attribs({"class": self.get_field_value('filebutton_class')})
-
-
-        # set the label 'for' attribute
-        self[1][0][0].update_attribs({'for':self[1][0][1].get_id()})
+        if self.wf.required:
+            self[1][0][1].attribs["required"] = "required"
+        if self.wf.filebutton_class:
+            self[1][0][1].attribs["class"] = self.wf.filebutton_class
 
         # submit button
         # the div holding submit label and button
-        if self.get_field_value('submitinputdiv_class'):
-            self[1][1].attribs = {"class": self.get_field_value('submitinputdiv_class')}
-        if self.get_field_value('submitbutton_label'):
-            self[1][1][0][0] = self.get_field_value('submitbutton_label')
-        if self.get_field_value('submitbuttonlabel_class'):
-            self[1][1][0].attribs = {"class": self.get_field_value('submitbuttonlabel_class')}
-        if self.get_field_value('submitbuttonlabel_style'):
-            self[1][1][0].update_attribs({"style": self.get_field_value('submitbuttonlabel_style')})
+        if self.wf.submitinputdiv_class:
+            self[1][1].attribs["class"] = self.wf.submitinputdiv_class
+        if self.wf.submitbutton_label:
+            self[1][1][0][0] = self.wf.submitbutton_label
+
+        self[1][1][0].set_class_style(self.wf.submitbuttonlabel_class, self.wf.submitbuttonlabel_style)
 
         # submit button
-        self[1][1][1].update_attribs({"value":self.get_field_value('submitbutton_text')})
+        self[1][1][1].attribs["value"] = self.wf.submitbutton_text
         # set an id in the submit button for the 'label for' tag
-        self[1][1][1].insert_id()
+        # and set the label 'for' attribute
+        self[1][1][0].attribs['for'] = self[1][1][1].insert_id()
 
-        if self.get_field_value('submitbutton_class'):
-            self[1][1][1].update_attribs({"class": self.get_field_value('submitbutton_class')})
-
-
-        # set the label 'for' attribute
-        self[1][1][0].update_attribs({'for':self[1][1][1].get_id()})
+        if self.wf.submitbutton_class:
+            self[1][1][1].attribs["class"] = self.wf.submitbutton_class
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -193,7 +186,8 @@ class UploadFile1(Widget):
         inputdiv_class: The css class of the div containing the label and input field
         error_class: The class applied to the paragraph containing the error message on error.
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # error div at 0
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -208,29 +202,27 @@ class UploadFile1(Widget):
 
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        if self.wf.error_class:
+            self[0].attribs["class"] = self.wf.error_class
         if self.error_status:
-            self[0].del_one_attrib("style")
+            del self[0].attribs["style"]
 
         # file button
         # the div holding file label and button
-        if self.get_field_value('inputdiv_class'):
-            self[1].attribs = {"class": self.get_field_value('inputdiv_class')}
-        if self.get_field_value('label'):
-            self[1][0][0] = self.get_field_value('label')
-        if self.get_field_value('label_class'):
-            self[1][0].attribs = {"class": self.get_field_value('label_class')}
-        if self.get_field_value('label_style'):
-            self[1][0].attribs = {"style": self.get_field_value('label_style')}
+        if self.wf.inputdiv_class:
+            self[1].attribs["class"] = self.wf.inputdiv_class
+        if self.wf.label:
+            self[1][0][0] = self.wf.label
+
+        self[1][0].set_class_style(self.wf.label_class, self.wf.label_style)
 
         # set an id in the file button for the 'label for' tag
-        self[1][1].insert_id()
-        self[1][1].update_attribs({"name":self.get_formname('inputname')})
-        if self.get_field_value('filebutton_class'):
-            self[1][1].update_attribs({"class": self.get_field_value('filebutton_class')})
-        # set the label 'for' attribute
-        self[1][0].update_attribs({'for':self[1][1].get_id()})
+        # and set the label 'for' attribute
+        self[1][0].attribs['for'] = self[1][1].insert_id()
+
+        self[1][1].attribs["name"] = self.get_formname('inputname')
+        if self.wf.filebutton_class:
+            self[1][1].attribs["class"] = self.wf.filebutton_class
 
 
     @classmethod
@@ -300,7 +292,8 @@ class SubmitUploadFile2(Widget):
         hidden_field3: A third hidden field value, leave blank if unused, name used as the get field name
         hidden_field4: A fourth hidden field value, leave blank if unused, name used as the get field name
         """
-        Widget.__init__(self, name=name, tag_name="div", brief=brief, **field_args)
+        Widget.__init__(self, name=name, brief=brief, **field_args)
+        self.tag_name = "div"
         # hidden error
         self[0] = tag.Part(tag_name="div", attribs={"style":"display:none;"})
         self[0][0] = tag.Part(tag_name="p")
@@ -329,80 +322,72 @@ class SubmitUploadFile2(Widget):
     def _build(self, page, ident_list, environ, call_data, lang):
         "build the form"
         # Hides widget if no error and hide is True
-        self.widget_hide(self.get_field_value("hide"))
-        if self.get_field_value('error_class'):
-            self[0].update_attribs({"class":self.get_field_value('error_class')})
+        self.widget_hide(self.wf.hide)
+        if self.wf.error_class:
+            self[0].attribs["class"] = self.wf.error_class
         if self.error_status:
-            self[0].del_one_attrib("style")
-        if not self.get_field_value("action"):
+            del self[0].attribs["style"]
+        if not self.wf.action:
             # setting self._error replaces the entire tag
             self._error = "Warning: No form action"
             return
-        actionurl = skiboot.get_url(self.get_field_value("action"),  proj_ident=page.proj_ident)
+        actionurl = self.get_url(self.wf.action)
         if not actionurl:
             # setting self._error replaces the entire tag
             self._error = "Warning: broken link"
             return
         # update the action of the form
-        self[1].update_attribs({"action": actionurl})
+        self[1].attribs["action"] = actionurl
 
         # file button
         # the div holding file label and button
-        if self.get_field_value('fileinputdiv_class'):
-            self[1][0].attribs = {"class": self.get_field_value('fileinputdiv_class')}
-        if self.get_field_value('filebutton_label'):
-            self[1][0][0][0] = self.get_field_value('filebutton_label')
-        if self.get_field_value('filebuttonlabel_class'):
-            self[1][0][0].attribs = {"class": self.get_field_value('filebuttonlabel_class')}
+        if self.wf.fileinputdiv_class:
+            self[1][0].attribs["class"] = self.wf.fileinputdiv_class
+        if self.wf.filebutton_label:
+            self[1][0][0][0] = self.wf.filebutton_label
+        if self.wf.filebuttonlabel_class:
+            self[1][0][0].attribs["class"] = self.wf.filebuttonlabel_class
 
         # set an id in the file button for the 'label for' tag
-        self[1][0][1].insert_id()
+        # and set the label 'for' attribute
+        self[1][0][0].attribs['for'] = self[1][0][1].insert_id()
 
-        self[1][0][1].update_attribs({"name":self.get_formname('action')})
-
-        if self.get_field_value('required'):
-            self[1][0][1].update_attribs({"required":"required"})
-        if self.get_field_value('filebutton_class'):
-            self[1][0][1].update_attribs({"class": self.get_field_value('filebutton_class')})
-
-        # set the label 'for' attribute
-        self[1][0][0].update_attribs({'for':self[1][0][1].get_id()})
+        self[1][0][1].attribs["name"] = self.get_formname('action')
 
+        if self.wf.required:
+            self[1][0][1].attribs["required"] = "required"
+        if self.wf.filebutton_class:
+            self[1][0][1].attribs["class"] = self.wf.filebutton_class
 
         # the div holding the container
-        if self.get_field_value('container_class'):
-            self[1][1].attribs = {"class": self.get_field_value('container_class')}
-
+        if self.wf.container_class:
+            self[1][1].attribs["class"] = self.wf.container_class
 
         # submit button
         # the div holding submit label and button
-        if self.get_field_value('submitinputdiv_class'):
-            self[1][2].attribs = {"class": self.get_field_value('submitinputdiv_class')}
-        if self.get_field_value('submitbutton_label'):
-            self[1][2][0][0] = self.get_field_value('submitbutton_label')
-        if self.get_field_value('submitbuttonlabel_class'):
-            self[1][2][0].attribs = {"class": self.get_field_value('submitbuttonlabel_class')}
-        if self.get_field_value('submitbuttonlabel_style'):
-            self[1][2][0].update_attribs({"style": self.get_field_value('submitbuttonlabel_style')})
+        if self.wf.submitinputdiv_class:
+            self[1][2].attribs["class"] = self.wf.submitinputdiv_class
+        if self.wf.submitbutton_label:
+            self[1][2][0][0] = self.wf.submitbutton_label
+
+        self[1][2][0].set_class_style(self.wf.submitbuttonlabel_class, self.wf.submitbuttonlabel_style)
 
         # set an id in the submit button for the 'label for' tag
-        self[1][2][1].insert_id()
+        # and set the label 'for' attribute
+        submitid = self[1][2][1].insert_id()
+        self[1][2][0].attribs['for'] = submitid
 
-        if self.get_field_value('submitbutton_class'):
-            self[1][2][1].update_attribs({"class": self.get_field_value('submitbutton_class')})
+        if self.wf.submitbutton_class:
+            self[1][2][1].attribs["class"] = self.wf.submitbutton_class
 
         # set the name of the submit button, so its value will be submitted with the form
-        self[1][2][1].update_attribs({"name":self.get_formname('submitbutton')})
+        self[1][2][1].attribs["name"] = self.get_formname('submitbutton')
 
         # set the submitbutton contents
-        self[1][2][1][0] = self.get_field_value('submitbutton')
-
-        # set the label 'for' attribute
-        self[1][2][0].update_attribs({'for':self[1][2][1].get_id()})
+        self[1][2][1][0] = self.wf.submitbutton
 
         # set onchange string in 101  this sets the filename as the button value
-        _onchange = f'document.getElementById("{self[1][2][1].get_id()}").value=this.value'
-        self[1][0][1].update_attribs({"onchange": _onchange})
+        self[1][0][1].attribs["onchange"] = f'document.getElementById("{submitid}").value=this.value'
 
         # add ident and four hidden fields
         self.add_hiddens(self[1], page)
@@ -412,13 +397,11 @@ class SubmitUploadFile2(Widget):
         """Sets a submit event handler"""
         # this ensures any input text widgets added to the container, get local validation
         # when the form is submitted
-        jscript = """  $('#{ident} form').on("submit", function(e) {{
+        ident = self.get_id()
+        return f"""  $('#{ident} form').on("submit", function(e) {{
     SKIPOLE.widgets['{ident}'].eventfunc(e);
     }});
-""".format(ident=self.get_id())
-        return jscript
-
-
+"""
 
 
     @classmethod
