@@ -10,9 +10,9 @@ SKIPOLE.skiprefresh = false;
 SKIPOLE.restorepagepos = function() {
     // restore window position from last position stored
     if(typeof(Storage) !== "undefined") {
-        var pageidentarray = SKIPOLE.identdata.split('_');
-        var x_pos = pageidentarray[0] + '_' + pageidentarray[1] + '_x';
-        var y_pos = pageidentarray[0] + '_' + pageidentarray[1] + '_y';
+        let pageidentarray = SKIPOLE.identdata.split('_');
+        let x_pos = pageidentarray[0] + '_' + pageidentarray[1] + '_x';
+        let y_pos = pageidentarray[0] + '_' + pageidentarray[1] + '_y';
         if (sessionStorage.getItem(y_pos)) {
             // Restore the y position of the window
             $(window).scrollTop(parseInt(sessionStorage.getItem(y_pos)));
@@ -52,8 +52,8 @@ SKIPOLE.inallowedlist =  function (item, allowed_values) {
     if (allowed_values.length === 0) {
         return false;
         }
-    var regstring = '';
-    for (i = 0; i < allowed_values.length; i++) {
+    let regstring = '';
+    for (let i = 0; i < allowed_values.length; i++) {
         if (i < allowed_values.length -1) {
             regstring += '^' + allowed_values[i] + "$|";
             }
@@ -61,7 +61,7 @@ SKIPOLE.inallowedlist =  function (item, allowed_values) {
             regstring += '^' + allowed_values[i] + "$";
             }
         }
-    var reg = new RegExp(regstring);
+    let reg = new RegExp(regstring);
     return reg.test(item);
     }
 
@@ -72,15 +72,15 @@ SKIPOLE.setfields = function(result) {
        // result is the contents of a json page listing widgfields and values
        if ("ident_data" in result) {
            // set the value of SKIPOLE.identdata
-           var oldidentdata = SKIPOLE.identdata;
-           var pageidentarray = SKIPOLE.identdata.split('_');
+           let oldidentdata = SKIPOLE.identdata;
+           let pageidentarray = SKIPOLE.identdata.split('_');
            // set identdata to projectname_pagenumber_ident_data
            SKIPOLE.identdata = pageidentarray[0] + '_' + pageidentarray[1] + '_' + result["ident_data"];
            // set this in all hidden input 'ident' fields 
            $("input[name='ident']:hidden").attr("value", SKIPOLE.identdata);
            // and in all url get 'ident' fields
-           var old_string = "?ident=" + oldidentdata
-           var new_string = "?ident=" + SKIPOLE.identdata
+           let old_string = "?ident=" + oldidentdata
+           let new_string = "?ident=" + SKIPOLE.identdata
            $("a[href*='?ident=']").each(function(){
                // Update the ?ident= part of the url to the new ident_data string
                $(this).attr('href',$(this).attr('href').replace(old_string,new_string));
@@ -128,14 +128,14 @@ SKIPOLE.setfields = function(result) {
        if ("sessionStorage" in result) {
            // set the session storage data
             if (typeof(Storage) !== "undefined") {
-                for (var key in result["sessionStorage"]) {
+                for (const key in result["sessionStorage"]) {
                    sessionStorage.setItem(key, result["sessionStorage"][key]);
                    }
                // updates the value in any inputforms.HiddenSessionStorage widget
                if ( SKIPOLE.widget_register.hasOwnProperty("inputforms.HiddenSessionStorage") ) {
-                    var hsswidgets = SKIPOLE.widget_register["inputforms.HiddenSessionStorage"].length;
+                    let hsswidgets = SKIPOLE.widget_register["inputforms.HiddenSessionStorage"].length;
                     // for each HiddenSessionStorage registered, call its updatefunc method
-                    for (hsswidget = 0; hsswidget < hsswidgets; hsswidget++) {
+                    for (let hsswidget = 0; hsswidget < hsswidgets; hsswidget++) {
                         // for each hsswidget in the register
                         SKIPOLE.widgets[SKIPOLE.widget_register["inputforms.HiddenSessionStorage"][hsswidget]].updatefunc();
                         }
@@ -145,14 +145,14 @@ SKIPOLE.setfields = function(result) {
        if ("localStorage" in result) {
            // set the local storage data
             if (typeof(Storage) !== "undefined") {
-                for (var key in result["localStorage"]) {
+                for (const key in result["localStorage"]) {
                    localStorage.setItem(key, result["localStorage"][key]);
                    }
                // updates the value in any inputforms.HiddenLocalStorage widget
                if ( SKIPOLE.widget_register.hasOwnProperty("inputforms.HiddenLocalStorage") ) {
-                    var hlswidgets = SKIPOLE.widget_register["inputforms.HiddenLocalStorage"].length;
+                    let hlswidgets = SKIPOLE.widget_register["inputforms.HiddenLocalStorage"].length;
                     // for each HiddenLocalStorage registered, call its updatefunc method
-                    for (hlswidget = 0; hlswidget < hlswidgets; hlswidget++) {
+                    for (let hlswidget = 0; hlswidget < hlswidgets; hlswidget++) {
                         // for each hlswidget in the register
                         SKIPOLE.widgets[SKIPOLE.widget_register["inputforms.HiddenLocalStorage"][hlswidget]].updatefunc();
                         }
@@ -162,9 +162,9 @@ SKIPOLE.setfields = function(result) {
        if ("environ" in result) {
            // displays the environ in any debug_tools.ShowEnviron widget
            if ( SKIPOLE.widget_register.hasOwnProperty("debug_tools.ShowEnviron") ) {
-                var showenvirons = SKIPOLE.widget_register["debug_tools.ShowEnviron"].length;
+                let showenvirons = SKIPOLE.widget_register["debug_tools.ShowEnviron"].length;
                 // for each ShowEnviron registered, call its updatefunc method
-                for (showenviron = 0; showenviron < showenvirons; showenviron++) {
+                for (let showenviron = 0; showenviron < showenvirons; showenviron++) {
                     // for each showenviron in the register
                     SKIPOLE.widgets[SKIPOLE.widget_register["debug_tools.ShowEnviron"][showenviron]].updatefunc(result["environ"]);
                     }
@@ -173,9 +173,9 @@ SKIPOLE.setfields = function(result) {
        if ("call_data" in result) {
            // displays the call_data dictionary in any debug_tools.ShowCallData widget
            if ( SKIPOLE.widget_register.hasOwnProperty("debug_tools.ShowCallData") ) {
-                var showthem = SKIPOLE.widget_register["debug_tools.ShowCallData"].length;
+                let showthem = SKIPOLE.widget_register["debug_tools.ShowCallData"].length;
                 // for each ShowCallData registered, call its updatefunc method
-                for (showit = 0; showit < showthem; showit++) {
+                for (let showit = 0; showit < showthem; showit++) {
                     // for each showenviron in the register
                     SKIPOLE.widgets[SKIPOLE.widget_register["debug_tools.ShowCallData"][showit]].updatefunc(result["call_data"]);
                     }
@@ -184,9 +184,9 @@ SKIPOLE.setfields = function(result) {
        if ("ident_list" in result) {
            // displays the identlist in any debug_tools.ShowResponders widget
            if ( SKIPOLE.widget_register.hasOwnProperty("debug_tools.ShowResponders") ) {
-                var showresponders = SKIPOLE.widget_register["debug_tools.ShowResponders"].length;
+                let showresponders = SKIPOLE.widget_register["debug_tools.ShowResponders"].length;
                 // for each ShowResponders registered, call its updatefunc method
-                for (showresponder = 0; showresponder < showresponders; showresponder++) {
+                for (let showresponder = 0; showresponder < showresponders; showresponder++) {
                     // for each showresponder in the register
                     SKIPOLE.widgets[SKIPOLE.widget_register["debug_tools.ShowResponders"][showresponder]].updatefunc(result["ident_list"]);
                     }
@@ -221,15 +221,15 @@ SKIPOLE.setfields = function(result) {
              $("body").removeAttr("class");
              }
          }
-       var widg_fields = {};
+       let widg_fields = {};
        // widg_fields will be a dictionary of widget id: list of json set fields
-       for (widgfield in result) {
+       for (const widgfield in result) {
            // widgfield parameters
            if (!widgfield.includes(":")) {
                continue;
                }
-           var splitwidgfield = widgfield.split(":");
-           var widg_id = splitwidgfield[0];
+           let splitwidgfield = widgfield.split(":");
+           let widg_id = splitwidgfield[0];
 
            if (widg_id in SKIPOLE.widgets) {
                if (!(widg_id in widg_fields)) {
@@ -249,7 +249,7 @@ SKIPOLE.setfields = function(result) {
                        }
                    }
                if (splitwidgfield[1] == "hide") {
-                   var sectionpart = $("#"+widg_id);
+                   let sectionpart = $("#"+widg_id);
                    if (result[widgfield]) {                      
                        if (sectionpart.is(":visible")) {
                            sectionpart.fadeOut('slow');
@@ -267,10 +267,10 @@ SKIPOLE.setfields = function(result) {
             }
 
         // Now set the widget values, after first setting widget_class and widget_style - which all widgets have
-        for (widg_ident in widg_fields) {
-            var fieldlist = widg_fields[widg_ident];
-            var thiswidget = SKIPOLE.widgets[widg_ident]
-            var widget_class = thiswidget.fieldarg_in_result('widget_class', result, fieldlist);
+        for (const widg_ident in widg_fields) {
+            let fieldlist = widg_fields[widg_ident];
+            let thiswidget = SKIPOLE.widgets[widg_ident]
+            let widget_class = thiswidget.fieldarg_in_result('widget_class', result, fieldlist);
             if (widget_class != undefined) {
                 if (widget_class) {
                     $("#"+widg_ident).attr("class", widget_class);
@@ -279,7 +279,7 @@ SKIPOLE.setfields = function(result) {
                     $("#"+widg_ident).removeAttr("class");
                     }
                 }
-            var widget_style = thiswidget.fieldarg_in_result('widget_style', result, fieldlist);
+            let widget_style = thiswidget.fieldarg_in_result('widget_style', result, fieldlist);
             if (widget_style != undefined) {
                 if (widget_style) {
                     $("#"+widg_ident).attr("style", widget_style);
@@ -306,28 +306,28 @@ SKIPOLE.setfields = function(result) {
 // for an input text widgfield, call its validators
 // this is called by widgets that submit a text input field
 SKIPOLE.validate = function(widgfield_name, widgfield_value) {
-    var widgparts = widgfield_name.split(":");
-    var field_no_i = widgparts[1].split("-")[0];
-    var name_no_i = widgparts[0] + ":" + field_no_i;
+    let widgparts = widgfield_name.split(":");
+    let field_no_i = widgparts[1].split("-")[0];
+    let name_no_i = widgparts[0] + ":" + field_no_i;
     if (!(name_no_i in SKIPOLE.validators)) {
        return [true, '', ''];
        };
-    var val_list = SKIPOLE.validators[name_no_i];
-    var count = val_list.length;
+    let val_list = SKIPOLE.validators[name_no_i];
+    let count = val_list.length;
     if (count === 0) {
        return [true, '', ''];
        };
-    for(var i = 0; i < count; i++) {
-       var val = val_list[i];
-       var allowed_values = val[3];
+    for(let i = 0; i < count; i++) {
+       let val = val_list[i];
+       let allowed_values = val[3];
        if (!(widgfield_value in allowed_values)) {
-           var module = val[0];
-           var valclass = val[1];
-           var error_message = val[2];
-           var args = val[4];
-           var validator = SKIPOLE[module][valclass];
-           var result = validator(widgfield_value, allowed_values, args);
-           var default_error_widget = SKIPOLE.default_error_widget;
+           let module = val[0];
+           let valclass = val[1];
+           let error_message = val[2];
+           let args = val[4];
+           let validator = SKIPOLE[module][valclass];
+           let result = validator(widgfield_value, allowed_values, args);
+           let default_error_widget = SKIPOLE.default_error_widget;
            if (!result) {
                if (val[5]) {
                    return [false, error_message, val[5]];
@@ -344,15 +344,15 @@ SKIPOLE.validate = function(widgfield_name, widgfield_value) {
 // return true if ok, false if not.  If false - validation failed, set input_errored_class
 // into input fields, and display validation error messages
 SKIPOLE.form_validate = function(selected_form) {
-    var all_ok = true;
+    let all_ok = true;
     selected_form.find('input[type="text"], input[type="password"]').each(function() {
         // for each text input field, get its value and call any validator functions
-        var text_input = $(this);
+        let text_input = $(this);
         if (text_input.prop('disabled')) {
             return true;
             }
         // get the widget of this input field
-        var input_widg_id = text_input.prop('name').split(":")[0];
+        let input_widg_id = text_input.prop('name').split(":")[0];
         if (input_widg_id in SKIPOLE.widgets) {
             var input_widg = SKIPOLE.widgets[input_widg_id];
             }
@@ -362,7 +362,7 @@ SKIPOLE.form_validate = function(selected_form) {
         // error_status[0] is true if ok, false if not
         // error_status[1] is an optional error message from the widgfield
         // error_status[2] is the validator display widget
-        var error_status = SKIPOLE.validate(text_input.prop('name'), text_input.val());
+        let error_status = SKIPOLE.validate(text_input.prop('name'), text_input.val());
         if (error_status[0]) {
             // validated - remove input_errored_class
             if ("input_errored_class" in input_widg.fieldvalues) {
@@ -390,7 +390,7 @@ SKIPOLE.form_validate = function(selected_form) {
 
 // sets <br /> into the string and escapes other html characters 
 SKIPOLE.textbr = function(string) {
-    var entityMap = {
+    let entityMap = {
         "&": "&amp;",
         "<": "&lt;",
         ">": "&gt;",
@@ -409,7 +409,7 @@ SKIPOLE.textbr = function(string) {
 
 // Used when a request for a json file fails
 SKIPOLE.json_failed = function( jqXHR, textStatus, errorThrown ) {
-        var msg = '';
+        let msg = '';
         if (jqXHR.status === 0) {
             msg = 'Connection failure: Possible network problem.';
         } else if (jqXHR.status == 404) {
@@ -429,7 +429,7 @@ SKIPOLE.json_failed = function( jqXHR, textStatus, errorThrown ) {
        if (SKIPOLE.default_error_widget) {
            // create dictionary result, to be used instead of received json file
            // and flags an error message in the page default error widget
-           var result = {};
+           let result = {};
            result[SKIPOLE.default_error_widget + ":show_error"] = msg;
            SKIPOLE.setfields( result );
        } else {
@@ -474,14 +474,14 @@ SKIPOLE.BaseWidget.prototype.validate = function(fieldarg, fieldval) {
     if (!this.widg_id) {
         return [true, '', ''];
         }
-    var widgfield = this.widg_id + ':' + this.fieldname(fieldarg);
+    let widgfield = this.widg_id + ':' + this.fieldname(fieldarg);
     return SKIPOLE.validate(widgfield, fieldval);
     };
 
 SKIPOLE.BaseWidget.prototype.fieldarg_in_result = function (fieldarg, result, fieldlist) {
-    var fieldname = this.fieldname(fieldarg);
+    let fieldname = this.fieldname(fieldarg);
     if ($.inArray(fieldname, fieldlist) !== -1) {
-        var widgfield = this.widg_id + ":" + fieldname;
+        let widgfield = this.widg_id + ":" + fieldname;
         if (widgfield in result) {
             return result[widgfield];
             }
@@ -492,8 +492,8 @@ SKIPOLE.BaseWidget.prototype.set_attribute = function (attname, fieldarg, result
     if (!this.widg_id) {
         return;
         }
-    var the_widg = this.widg;
-    var attval = this.fieldarg_in_result(fieldarg, result, fieldlist);
+    let the_widg = this.widg;
+    let attval = this.fieldarg_in_result(fieldarg, result, fieldlist);
     if (attval != undefined) {
         if (attval) {
             the_widg.attr(attname, attval);
@@ -567,11 +567,11 @@ SKIPOLE.BaseWidget.prototype.sethiddenfields = function(fieldlist, result) {
 
 // given href, and a get field with new value, returns new href with changed get field
 SKIPOLE.BaseWidget.prototype.setgetfield = function(href, fieldarg, fieldvalue) {
-    var get_field_value =  encodeURIComponent( fieldvalue );
-    var get_field_name = this.formname(fieldarg);
+    let get_field_value =  encodeURIComponent( fieldvalue );
+    let get_field_name = this.formname(fieldarg);
     // fieldname may have : so when comparing with href parts, need encoded value
-    var encode_field_name = encodeURIComponent( get_field_name );
-    var qindex = href.indexOf('?');
+    let encode_field_name = encodeURIComponent( get_field_name );
+    let qindex = href.indexOf('?');
     if (qindex === -1) {
         if ( get_field_value === '') {
             return href;
@@ -580,11 +580,11 @@ SKIPOLE.BaseWidget.prototype.setgetfield = function(href, fieldarg, fieldvalue)
             return href + "?ident=" + SKIPOLE.identdata + "&" + get_field_name + "=" + get_field_value;
             }
         }
-    var url = href.substring(0, qindex);
-    var querystring = href.substring(qindex+1);
-    var ampindex = querystring.indexOf('&');
+    let url = href.substring(0, qindex);
+    let querystring = href.substring(qindex+1);
+    let ampindex = querystring.indexOf('&');
     if (ampindex === -1) {
-        var qv = querystring.split('=');
+        let qv = querystring.split('=');
         if ((qv[0] === encode_field_name) || (qv[0] === get_field_name)) {
             if ( get_field_value === '') {
                 return url;
@@ -602,11 +602,11 @@ SKIPOLE.BaseWidget.prototype.setgetfield = function(href, fieldarg, fieldvalue)
                 }
              }
         }
-    var queries = querystring.split("&");
-    var qlist = [];
-    var flag = false;
-    for (index = 0; index < queries.length; ++index) {
-        var qv = queries[index].split('=');
+    let queries = querystring.split("&");
+    let qlist = [];
+    let flag = false;
+    for (let index = 0; index < queries.length; ++index) {
+        let qv = queries[index].split('=');
         if ((qv[0] === encode_field_name) || (qv[0] === get_field_name)) {
             qv[1] = get_field_value;
             flag = true;
@@ -620,7 +620,7 @@ SKIPOLE.BaseWidget.prototype.setgetfield = function(href, fieldarg, fieldvalue)
             qlist.push( get_field_name + "=" + get_field_value);
             }
         }
-    var new_query = qlist.join("&");
+    let new_query = qlist.join("&");
     return url + "?" + new_query;
   };
 
@@ -629,8 +629,8 @@ SKIPOLE.BaseWidget.prototype.get_error = function (result) {
     if (!this.display_errors) {
         return;
         }
-    var fieldname = this.fieldname('show_error');
-    var widgfield = this.widg_id + ":" + fieldname;
+    let fieldname = this.fieldname('show_error');
+    let widgfield = this.widg_id + ":" + fieldname;
     if (!(widgfield in result)) {
         widgfield = this.widg_id + ":show_error";
         if (!(widgfield in result)) {
@@ -653,13 +653,13 @@ SKIPOLE.BaseWidget.prototype.check_error = function (fieldlist, result) {
     if (!this.display_errors) {
         return false;
         }
-    var error_message = this.get_error(result);
+    let error_message = this.get_error(result);
     if (error_message) {
         this.show_error(error_message);
         return true;
         }
     // check for clear_error
-    var clear = this.fieldarg_in_result('clear_error', result, fieldlist);
+    let clear = this.fieldarg_in_result('clear_error', result, fieldlist);
     if (clear) {
         this.clear_error();
         }
@@ -712,10 +712,10 @@ SKIPOLE.BaseWidget.prototype.set_errored = function (input_field, input_errored)
 
 SKIPOLE.BaseWidget.prototype.set_accepted_errored = function (input_field, fieldlist, result) {
     // input_accepted
-    var input_accepted = this.fieldarg_in_result('set_input_accepted', result, fieldlist);
+    let input_accepted = this.fieldarg_in_result('set_input_accepted', result, fieldlist);
     this.set_accepted(input_field, input_accepted);
     // input_errored
-    var input_errored = this.fieldarg_in_result('set_input_errored', result, fieldlist);
+    let input_errored = this.fieldarg_in_result('set_input_errored', result, fieldlist);
     this.set_errored(input_field, input_errored);
     };
 
@@ -751,11 +751,11 @@ SKIPOLE.BaseWidget.prototype.show_error = function (error_message) {
     if (!error_message) {
         error_message = "Unknown Error";
         }
-    var the_widg = this.widg;
+    let the_widg = this.widg;
     the_widg.attr("data-status", "error");
     // get error div, being first child
-    var error_div = the_widg.find(':first');
-    var error_para = error_div.find(':first');
+    let error_div = the_widg.find(':first');
+    let error_para = error_div.find(':first');
     error_para.text(error_message);
     if (!(error_div.is(":visible"))) {
        error_div.show();
@@ -767,11 +767,11 @@ SKIPOLE.BaseWidget.prototype.clear_error = function() {
     if (!this.display_errors) {
         return;
         }
-    var the_widg = this.widg;
+    let the_widg = this.widg;
     if (the_widg.attr("data-status") == "error") {
         the_widg.removeAttr( "data-status" );
         }
-    var error_div = the_widg.find(':first');
+    let error_div = the_widg.find(':first');
     if (error_div.is(":visible")) {
         error_div.hide();
         }
@@ -782,11 +782,11 @@ SKIPOLE.BaseWidget.prototype.clear_error = function() {
 SKIPOLE.clear_all_errors = function() {
     $('[data-status="error"]').each(function() {
         // for each widget
-        var thiselement = $(this);
+        let thiselement = $(this);
         // get the widget of this thiselement
-        var widg_id = thiselement.prop('id');
+        let widg_id = thiselement.prop('id');
         if (widg_id in SKIPOLE.widgets) {
-            var this_widg = SKIPOLE.widgets[widg_id];
+            let this_widg = SKIPOLE.widgets[widg_id];
             this_widg.clear_error();
             }
     })
